// TIPAfile.java
// $Id: TIPAfile.java 1161 2007-01-02 15:50:12Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
//
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.database.TIPA;

import at.iem.adaba.AdabaShared;
import at.iem.adaba.database.*;
import at.iem.tools.StringX;
import at.iem.tools.linguistic.IPA;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;


public class TIPAfile implements Exporter{

    private File file=null;

    public TIPAfile(File f) 
    {
	file=f;
    }

    
    /** export a database to a TIPA-file;
     * no assumption is made on the order of the exported database entries
     * @param source the database to be exported
     * @return <code>true</code> on success, <code>false</code> otherwise
     */
    public boolean exportFrom(TranscriptionDataBase source) throws Exception 
    {
	return exportFrom(source, source.getAllEntries());
    }


    /** export a selection of entries to a TIPA-file;
     * the order of the exported entries is the same as the order of the exportKeys
     * @param source the database to be exported
     * @param exportKeys the keys to be exported
     * @return <code>true</code> on success, <code>false</code> otherwise
     */
    public boolean exportFrom(TranscriptionDataBase source, java.lang.String[] exportKeys) throws Exception
    {
	at.iem.tools.Logger.println("exporting database to TIPA: "+exportKeys.length);
	try {
	    String encoding="ISO-8859-15";
	    FileOutputStream ostr = new FileOutputStream(file); 
	    OutputStreamWriter out = new OutputStreamWriter(ostr, encoding);
	    String lineEnd =  System.getProperty("line.separator");
	    out.write("\\documentclass{article}"+lineEnd+
		      "\\usepackage[latin1]{inputenc}"+lineEnd+
                      "\\usepackage[T1]{tipa}"+lineEnd+
                      "\\usepackage{tipx}"+lineEnd+lineEnd+
                      "\\def\\transcription#1#2{\\item[#1:] #2}"+lineEnd+
                      "\\newenvironment{transcriptionlist}[1]{\\paragraph{#1}\\begin{description}\\setlength{\\itemsep}{1pt}\\setlength{\\parskip}{0pt}\\setlength{\\parsep}{0pt}}{\\end{description}}"+lineEnd+lineEnd+
		      "\\begin{document}"+lineEnd+
		      "\\twocolumn"+lineEnd+
		      "\\title{adaba-export: "+file+"}"+lineEnd+
		      "\\author{}"+lineEnd+
		      "\\date{}"+lineEnd+lineEnd+lineEnd);

            for(int i=0; i<exportKeys.length; i++){
                try {
                    TranscriptionEintrag[]exportEintrags=source.lookup(exportKeys[i]);
                    if(null!=exportEintrags) {
                        for(int j=0; j<exportEintrags.length; j++) {
                            TranscriptionEintrag exportEintrag=exportEintrags[j];
                            
                            out.write("\\begin{transcriptionlist}{"+exportEintrag.getName()+"}"+lineEnd);
                            int ipa_length=AdabaShared.region.length;
                            boolean same=true;
                            IPA ipa0=exportEintrag.getIPA(AdabaShared.region[0][0]);
                            if(null==ipa0)ipa0=new IPA("");
                            for(int ipa_c=1; ipa_c<ipa_length; ipa_c++) {
                                if(!ipa0.equals(exportEintrag.getIPA(AdabaShared.region[ipa_c][0]))) {
                                    same=false;
                                    break;
                                }
                            }
                            try{
                                if(same) {
                                    export_transTIPA(out, ipa0, null);
                                } else 
                                    for(int ipa_c=0; ipa_c<ipa_length; ipa_c++) {
                                        IPA ipa=exportEintrag.getIPA(AdabaShared.region[ipa_c][0]);
                                        if(null!=ipa && !"".equals(ipa)){
                                            export_transTIPA(out, ipa, AdabaShared.region[ipa_c][0]);
                                            out.write(lineEnd);
                                        }
                                    }
                            } catch (Exception ex) {
                                at.iem.tools.Logger.println("tipa-transexport-exception for '"+exportKeys[i]+"': '"+ex+"'");

                            }
                            out.write("\\end{transcriptionlist}"+lineEnd);
                        }
                        out.write(lineEnd);
                    } else {
                        at.iem.tools.Logger.println("skipping empty entry for '"+exportKeys[i]+"'");
                    }
                } catch (Exception ex){
                    at.iem.tools.Logger.println("tipa-exportexception for '"+exportKeys[i]+"': '"+ex+"'");
                }
            }
            out.write("\\end{document}"+lineEnd);
            out.flush();
	} catch  (Exception ex) {
	    at.iem.tools.Logger.println("couldn't save database as TIPA...\n--> "+ex);
	    //AdabaMessages.showMessage(StringX.replace(AdabaShared.getLabel("ErrorSaveAs"), "%s", file.getName()));
            return false;
	}
	return true;
    }

    /**
     * write the a transcription to an OutputStreamWriter
     * @param out the OutputStreamWriter to write to
     * @param ipa the transcription
     * @param prefix a string that is prefixed to the TIPA-output in emphasised
     * @return <code>true</code> on success; <code>false</code> otherwise
     */
    private void export_transTIPA(OutputStreamWriter out, IPA ipa, String prefix)
	throws java.io.IOException{

        if(null==out)
            return;
        if(null==ipa || ipa.equals(""))
            return;

        ipa.fixDelimiters("[[", "]]");
	String tipa=ipa.getTIPA("[[", "]]");


        if(null==tipa)
            return;
        tipa=tipa.trim();
        if("".equals(tipa) || "[[]]".equals(tipa))
            return;

	tipa=StringX.replace(tipa, "[[", "\\textipa{");
	tipa=StringX.replace(tipa, "]]", "}");


        out.write("\\transcription{");
        if(prefix!=null&&!"".equals(prefix))
            out.write(prefix);

        out.write("}{"+tipa+"}");
    }
}
