// TranscriptionDataBase.java
// $Id: TranscriptionDataBase.java 1161 2007-01-02 15:50:12Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
//
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.database;


public interface TranscriptionDataBase {

    /* ================= metadata on database ================== */
    /**
     * get a list of all entries in the database
     * @return a sorted array of all keys in the database
     */
    String[]  getAllEntries() ;
    /**
     * get the number of entries in the database
     * @return the number of all keys in the database
     */
    int  getNumEntries() ;


    /**
     * get a short name of the database
     * @return a short "Name"
     */
    String  getName() ;

    /**
     * get the full connection information for the database
     * @return the URI-encoded connection string of the database
     */
    java.net.URI  getURI() ;


    String  toText() ;


    /* ================= interace with files ================= */

    /* LATER remove this as it is database-specific */

    boolean  load(java.io.File fil) ;
    boolean  load(String fil) ;
    void     save() ;




    /* ================== add/remove/change entries =================== */


    /**
     * start asynchronous database modifications;
     * this can be used in order to do some modifications without having to update 
     * auxiliary structures (which might eventually take a lot of time); 
     * if the backend prefers it is free to stay in synchronous mode;
     * THINK: should databases be locked in asynchronous mode?
     * @see #stopAsync() stopAsync
     */
    void startAsync();

    /**
     * add a new TranscriptionEintrag into the database (if the key already exists, create an alternative one)
     * @param entry the TranscriptionEintrag to be inserted
     * @return the (possibly modified) key of the added entry on success, <code>null</code> if adding failed
     */
    String add(TranscriptionEintrag entry) ;


    /**
     * add a new TranscriptionEintrag into the database (if the key already exists, create an alternative one)
     * @param oldkey the original key used for identifying the TranscriptionEintrag
     * @param newkey the new key of the TranscriptionEintrag identified by oldkey
     * @return the (new) key of the added entry on success, <code>null</code> if adding failed
     */
    String rename(String oldkey, String newkey) ;

    /**
     * merge a TranscriptionEintrag into an already existing one (given via a key)
     * if the key does not exist, a new TranscriptionEintrag is made (with this key)
     * LATER add a pattern on which entries are to be imported
     * @param key the key of the TranscriptionEintrag into which we merge
     * @param entry the data to be merged into the existing entry
     * @return the merged entry, <code>null</code> if merging failed
     */
    TranscriptionEintrag merge(String key, TranscriptionEintrag entry);

    /**
     * delete a TranscriptionEintrag from the database
     * @param key the key to the TranscriptionEintrag to be deleted
     * @return <code>true</code> on success, <code>false</code> otherwise
     */
    boolean  delete(String key) ;
    /**
     * delete a number of entries denoted by their keys
     * @param keys an Array of keys
     */
    void  delete(String[] keys) ;

    /**
     * stop asynchronous database modifications;
     * calling this will allow the database to synchronize itself to the backend
     * @see #startAsync() startAsync
     */
    void stopAsync();





    /* ============== database querying ==================== */

    /**
     * lookup a key
     * @param key a key to lookup in the database
     * @return an array of 1 TranscriptionEintrag  corresponding to the given key
     */
    TranscriptionEintrag[]  lookup(String key) ;
    /**
     * lookup an array of Strings (several keys)
     * @param key an array of keys to lookup in the database
     * @return an array of TranscriptionEintrag, each entry corresponding to the given entry in the key[]
     */
    TranscriptionEintrag[]  lookup(String[] key) ;

    /**
     * lookup all entries that match a given pattern
     * @param pattern a EintragPattern to search for
     * @return an array of keys which match <code>pattern</code>
     */
    String[]  search(EintragPattern pattern) throws Exception;

    /**
     * lookup all entries that start with a given pattern
     * @param pattern a pattern to search for
     * @return an array TranscriptionEintrag's the keys of which start with <code>pattern</code>
     * @see #startsWithS(String) startsWithS
     */
    String[]  startsWith(String pattern) ;

    /**
     * lookup all entries that start with a given pattern
     * @param pattern a pattern to search for
     * @return an array keys that start with <code>pattern</code>
     * @see #startsWith(String) startsWith
     */
    String[]  isExactly(String pattern) ;





    /**
     * stp the search (if threaded)
     * LATER think MORE about threads!!!
     */
    void stopSearch();













    /* =============== various I/O-routines ================= */
    /* LATER move this into separate classes */

    /**
     * import a database;
     * <code>mode</code> controls, how to handle entries, that appear in both the new and the old database
     * <table>
     * <tr><th>mode</th><th>action</th></tr>
     * <tr><td>0</td><td>never overwrite existing entries</td></tr>
     * <tr><td>1</td><td>always overwrite existing entries</td></tr>
     * <tr><td>2</td><td>overwrite existing entries only if the timestamp is newer</td></tr>
     * </table>
     * @param dbfile the database-file to import
     * @param editedOnly if set to <code>true</code> only entries with the flag "edited" are imported
     * @param mode how to handle clashing entries
     * @param storeAsAlternate whether entries that are rejected because of clashes are stored as "alternatives"
     * @return a String[] array of unresolved Entries; <code>null</code> if no clashes occured
     */
    String[]  importATD(java.io.File[] dbfile, boolean editedOnly, int mode, boolean storeAsAlternate) ;

    /**
     * import a database;
     * <code>mode</code> controls, how to handle entries, that appear in both the new and the old database
     * <table>
     * <tr><th>mode</th><th>action</th></tr>
     * <tr><td>0</td><td>never overwrite existing entries</td></tr>
     * <tr><td>1</td><td>always overwrite existing entries</td></tr>
     * <tr><td>2</td><td>overwrite existing entries only if the timestamp is newer</td></tr>
     * </table>
     * @param dbfile the database-files to import
     * @param editedOnly if set to <code>true</code> only entries with the flag "edited" are imported
     * @param mode how to handle clashing entries
     * @param storeAsAlternate whether entries that are rejected because of clashes are stored as "alternatives"
     * @return an array of unresolved Entries; <code>null</code> if no clashes occured
     */
    String[]  importATD(java.io.File dbfile, boolean editedOnly, int mode, boolean storeAsAlternate) ;
    /**
     * export the data to a new database;
     * this includes updating all entries to the newest format
     */
    void  exportATD(java.io.File fil) ;
    void  exportATD(java.io.File fil, String[] exportentries) ;


    /* ---------------- SAMPA --------------- */
    void  importSAMPA(java.io.File fil) ;
    void  importSAMPA(java.io.File fil, String regionID) ;
    void  importSAMPA(String filname) ;
    void  importSAMPA(String filname, String regionID) ;

    /**
     * export the data to a SAMPA-file
     */
    void  exportSAMPA(java.io.File fil, String regionID) ;
    void  exportSAMPA(java.io.File fil, String[] exportentries, String regionID) ;


    /* ---------------- TXT --------------- */
    /**
     * export the data from a txt-file (UTF-8 encoded)
     */
    void  importTXT(java.io.File fil) ;
    void  importTXT(String filname) ;
}
