// ElanTranscription.java
// $Id: ElanTranscription.java 1 2006-09-11 13:22:49Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
//
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.elan;
import java.util.ArrayList;

/**
 * hlt eine leichte Struktur (wie in C) mit den werten
 * text (orthographisch oder phonetisch oder...)
 * startPosition, stopPosition (position in <ms> im soundfile)
 * startCharacter, stopCharacter (position im gesamttext)
 */


public class ElanTranscription {

    class ElanChunk {
	public String data;
	public int   stopTime;
	public int   stopPosition;
        
	public ElanChunk(String data, int stoptime, int stopcharacter) {
            this.data=data;
            this.stopTime=stoptime;
            this.stopPosition=stopcharacter;
	}
    }

    private ArrayList<ElanChunk> chunks = new ArrayList<ElanChunk>();

    public ElanTranscription() {
        // nothing to do...
    }

    /**
     * append a new transcription-chunk to the end of the transcription.
     * the startTime of the transcription is implied by the stopTime of the previous chunk
     * the stopTime most not be less than the stopTime of the previous chunk
     * @param data the transcription itself
     * @param stopTime the stop-time (in ms) of the transcription
     * @return <code>true</code> on success, <code>false</code> otherwise
     */
    public boolean append(String data, int stopTime) {
        data=at.iem.tools.StringX.trimBlanks(data)+" ";
        int len=chunks.size();
        int lastTime=0;
        int lastPosition=0;
        if(len>0) {
            ElanChunk lastChunk=chunks.get(len-1);
            lastTime=lastChunk.stopTime;
            lastPosition=lastChunk.stopPosition;
        }
        if(lastTime>=stopTime) {
            // stopTime of previous chunk is bigger than ours!
            return false;
        }
        ElanChunk chunk = new ElanChunk(data, stopTime, lastPosition+data.length());
        return chunks.add(chunk);
    }

    /**
     * get the entire transcription
     * @return the transcription as a String
     */
    public String getText() {
        return getText(0,chunks.size());
    }

    /**
     * get the transcription of an enumerated data-chunk
     * @param id the ID of the data-chunk 
     * @return the transcription as a String
     */
    public String getText(int id) {
        if((id<0)||(id>=chunks.size()))
            return null;
        ElanChunk chunk = chunks.get(id);
        return new String(chunk.data);
    }

    /**
     * get the transcription of a series of enumerated data-chunks
     * @param startId the ID of the first data-chunk 
     * @param stopId the ID of the last data-chunk 
     * @return the transcription as a String
     */
    public String getText(int startId, int stopId) {
        if(chunks.size()==0)
            return null;
        if(startId<0)startId=0;
        if(stopId>chunks.size())stopId=chunks.size();
        String fulltext=new String();
        for (int i=startId; i<stopId;i++)
            fulltext=fulltext+(chunks.get(i).data);
        return fulltext;
    }

    /**
     * get the start-time (<ms>) of a data-chunk
     * @param id the ID of the data-chunk 
     * @return the start-time of the data-chunk
     */
    public int getStartTime(int id) {
        if((id<0)||(id>=chunks.size()))
            return -1;
        if(id==0)return 0;
        ElanChunk chunk = chunks.get(id-1);
        return chunk.stopTime+1;
    }
    /**
     * get the stop-time (<ms>) of a data-chunk
     * @param id the ID of the data-chunk 
     * @return the stop-time of the data-chunk
     */
    public int getStopTime(int id) {
        if((id<0)||(id>=chunks.size()))
            return -1;
        ElanChunk chunk = chunks.get(id);
        return chunk.stopTime;
    }

    /**
     * get the length of the data in <ms>
     * @return the length of the data-chunk
     */
    public int getStopTime() {
        if(chunks.size()<1)
            return -1;
        ElanChunk chunk = chunks.get(chunks.size()-1);
        return chunk.stopTime;
    }
    /**
     * get the start-position (in #characters) of a data-chunk
     * @param id the ID of the data-chunk 
     * @return the start-position (character) of the data-chunk
     */
    public int getStartPosition(int id) {
        if((id<0)||(id>=chunks.size()))
            return -1;
        if(id==0)return 0;
        ElanChunk chunk = chunks.get(id-1);
        return chunk.stopPosition;
    }

    /**
     * get the stop-position (in #characters) of a data-chunk
     * @param id the ID of the data-chunk 
     * @return the stop-position (characte) of the data-chunk
     */
    public int getStopPosition(int id) {
        if((id<0)||(id>=chunks.size()))
            return -1;
        ElanChunk chunk = chunks.get(id);
        return chunk.stopPosition-1;
    }
    /**
     * get the length of the data in characters
     * @return the length of the data-chunk
     */
    public int getStopPosition() {
        if(chunks.size()<1)
            return -1;
        ElanChunk chunk = chunks.get(chunks.size()-1);
        return chunk.stopPosition;
    }

    /**
     * get the ID of the chunk at time <time>
     * @param id the time to test for
     * @return the ID of the  data-chunk
     */
    public int timeToId(int time) {
        if(chunks.size()<1)return -1;
        if(time<0)return -1;
        // LATER make this use binary search...
        for(int i=0; i<chunks.size(); i++) {
            ElanChunk chunk = chunks.get(i);
            if(time<=chunk.stopTime) {
                return(i);
            }
        }
        return -1;
    }

    /**
     * get the ID of the chunk at character-position <pos>
     * @param id the character-position to test for
     * @return the ID of the  data-chunk
     */
    public int positionToId(int pos) {
        if(chunks.size()<1)return -1;
        if(pos<0)return -1;
        // LATER make this use binary search...
        for(int i=0; i<chunks.size(); i++) {
            ElanChunk chunk = chunks.get(i);
            if(pos<=chunk.stopPosition)
                return(i);
        }
        return -1;
    }

   /**
     * get the startTime of the chunk at character-position <pos>
     * @param id the character-position to test for
     * @return the startTime of the  data-chunk
     */
    public int startPositionToTime(int pos) {
        return getStartTime(positionToId(pos));
    }
   /**
     * get the stopTime of the chunk at character-position <pos>
     * @param id the character-position to test for
     * @return the stopTime of the  data-chunk
     */
    public int stopPositionToTime(int pos) {
        return getStopTime(positionToId(pos));
    }
   /**
     * get the start-position of the chunk at time <tim>
     * @param time the time of the chunk
     * @return the startPosition of the data-chunk
     */
    public int startTimeToPosition(int tim) {
        return getStartPosition(timeToId(tim));
    }
   /**
     * get the startTime of the chunk at character-position <pos>
     * @param  time the time of the chunk
     * @return the stopPosition of the data-chunk
     */
    public int stopTimeToPosition(int tim) {
        return getStopPosition(timeToId(tim));
    }

    /**
     * get the number of chunks
     * @return the number of chunks
     */
    public int length() {
        return chunks.size();
    }


    /**
     * print all chunks to the stdout
     */
    public void printChunks() {
        for(int i=0; i<chunks.size(); i++){
            System.out.println(""+chunks.get(i).data);
        }
    }
}
