// IPA.java
// $Id: IPA.java 1781 2007-06-26 08:10:52Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.tools.linguistic;

/**
 * Convert between IPA(unicode), SAMPA(ASCII), TIPA(TeX), ...<br>
 * IPA (International Phonetic Alphabet) is a system used for
 * describing something of phonetic importance (eg: pronunciation).
 * Unicode is fine for this purpose, because (most of) the IPA-glyphs
 * are part of the unicode-system.
 * Unfortunately typing unicode-symbols might prove hard on a normal ASCII-keyboard.
 * Various proposals have been made (i think) to do phonetic-transcription with
 * ASCII-based systems. One of those is SAMPA.<br>
 * Another problem is typesetting of IPA-Symbols.
 * The well-known and -liked un*x-typesetter (La)TeX has an extension
 * TIPA (TeX-IPA or Tokyo-IPA by Fukui Rei) that can be utilized.
 * so we provide various converters between IPA/unicode, SAMPA and TIPA.
 * <br>
 * This is the conversion table used:
 * <table align="center" border="1">
 *      <tr><td><b>unicode (hex)</b></td><td><b>IPA</b></td><td><b>SAMPA</b></td><td><b>TIPA</b></td></tr>
 * <tr><td>0x0020</td><td>&#x0020</td><td></td><td></td></tr>
 * <tr><td>0x0067</td><td>&#x0067</td><td>g\</td><td>\textg</td></tr>
 * <tr><td>0x00e6</td><td>&#x00e6</td><td>{</td><td>\ae</td></tr>
 * <tr><td>0x00e7</td><td>&#x00e7</td><td>C</td><td>\c{c}</td></tr>
 * <tr><td>0x00f0</td><td>&#x00f0</td><td>D</td><td>D</td></tr>
 * <tr><td>0x00f8</td><td>&#x00f8</td><td>2</td><td>\o</td></tr>
 * <tr><td>0x0127</td><td>&#x0127</td><td>X\</td><td>\textcrh</td></tr>
 * <tr><td>0x014b</td><td>&#x014b</td><td>N</td><td>N</td></tr>
 * <tr><td>0x0153</td><td>&#x0153</td><td>9</td><td>\oe</td></tr>
 * <tr><td>0x0180</td><td>&#x0180</td><td></td><td>\textcrb</td></tr>
 * <tr><td>0x0188</td><td>&#x0188</td><td></td><td>\texthtc</td></tr>
 * <tr><td>0x0195</td><td>&#x0195</td><td></td><td>\texthvlig</td></tr>
 * <tr><td>0x0199</td><td>&#x0199</td><td></td><td>\texthtk</td></tr>
 * <tr><td>0x019a</td><td>&#x019a</td><td></td><td>\textbarl</td></tr>
 * <tr><td>0x019b</td><td>&#x019b</td><td></td><td>\textcrlambda</td></tr>
 * <tr><td>0x019e</td><td>&#x019e</td><td>n\</td><td>\textnrleg</td></tr>
 * <tr><td>0x01a5</td><td>&#x01a5</td><td></td><td>\texthtp</td></tr>
 * <tr><td>0x01ab</td><td>&#x01ab</td><td></td><td>\textlhookt</td></tr>
 * <tr><td>0x01ad</td><td>&#x01ad</td><td></td><td>\texthtt</td></tr>
 * <tr><td>0x01b9</td><td>&#x01b9</td><td>Z\</td><td>\textrevyogh</td></tr>
 * <tr><td>0x01c0</td><td>&#x01c0</td><td>|\</td><td>|</td></tr>
 * <tr><td>0x01c1</td><td>&#x01c1</td><td>|\|\</td><td>||</td></tr>
 * <tr><td>0x01c2</td><td>&#x01c2</td><td>=\</td><td>\textdoublebarpipe</td></tr>
 * <tr><td>0x01c3</td><td>&#x01c3</td><td>!\</td><td>!</td></tr>
 * <tr><td>0x0250</td><td>&#x0250</td><td>6</td><td>5</td></tr>
 * <tr><td>0x0251</td><td>&#x0251</td><td>A</td><td>A</td></tr>
 * <tr><td>0x0252</td><td>&#x0252</td><td>Q</td><td>6</td></tr>
 * <tr><td>0x0253</td><td>&#x0253</td><td>b_&lt;</td><td>\!b</td></tr>
 * <tr><td>0x0254</td><td>&#x0254</td><td>O</td><td>O</td></tr>
 * <tr><td>0x0255</td><td>&#x0255</td><td>s\</td><td>C</td></tr>
 * <tr><td>0x0256</td><td>&#x0256</td><td>d'</td><td>\:d</td></tr>
 * <tr><td>0x0257</td><td>&#x0257</td><td>d_&lt;</td><td>\!d</td></tr>
 * <tr><td>0x0258</td><td>&#x0258</td><td>@\</td><td>9</td></tr>
 * <tr><td>0x0259</td><td>&#x0259</td><td>@</td><td>@</td></tr>
 * <tr><td>0x025a</td><td>&#x025a</td><td>@`</td><td>\textrhookschwa</td></tr>
 * <tr><td>0x025b</td><td>&#x025b</td><td>E</td><td>E</td></tr>
 * <tr><td>0x025c</td><td>&#x025c</td><td>3</td><td>\textrevepsilon</td></tr>
 * <tr><td>0x025d</td><td>&#x025d</td><td><i>3`</i></td><td>\textrhookrevepsilon</td></tr>
 * <tr><td>0x025e</td><td>&#x025e</td><td>3\</td><td>\textcloserevepsilon</td></tr>
 * <tr><td>0x025f</td><td>&#x025f</td><td>J\</td><td>\textbardotlessj</td></tr>
 * <tr><td>0x0260</td><td>&#x0260</td><td>g_&lt;</td><td>\!g</td></tr>
 * <tr><td>0x0261</td><td>&#x0261</td><td>g</td><td>g</td></tr>
 * <tr><td>0x0262</td><td>&#x0262</td><td>G\</td><td>\;G</td></tr>
 * <tr><td>0x0263</td><td>&#x0263</td><td>G</td><td>G</td></tr>
 * <tr><td>0x0264</td><td>&#x0264</td><td>7</td><td>7</td></tr>
 * <tr><td>0x0265</td><td>&#x0265</td><td>H</td><td>4</td></tr>
 * <tr><td>0x0266</td><td>&#x0266</td><td>h\</td><td>H</td></tr>
 * <tr><td>0x0267</td><td>&#x0267</td><td>x\</td><td>\texththeng</td></tr>
 * <tr><td>0x0268</td><td>&#x0268</td><td>1</td><td>1</td></tr>
 * <tr><td>0x0269</td><td>&#x0269</td><td>i\</td><td>\textiota</td></tr>
 * <tr><td>0x026a</td><td>&#x026a</td><td>I</td><td>I</td></tr>
 * <tr><td>0x026b</td><td>&#x026b</td><td>5 l_e</td><td>\|~l</td></tr>
 * <tr><td>0x026c</td><td>&#x026c</td><td>K</td><td>\textbeltl</td></tr>
 * <tr><td>0x026d</td><td>&#x026d</td><td>l'</td><td>\:l</td></tr>
 * <tr><td>0x026e</td><td>&#x026e</td><td>K\</td><td>\textlyoghlig</td></tr>
 * <tr><td>0x026f</td><td>&#x026f</td><td>M</td><td>W</td></tr>
 * <tr><td>0x0270</td><td>&#x0270</td><td>M\</td><td>\textturnmrleg</td></tr>
 * <tr><td>0x0271</td><td>&#x0271</td><td>F</td><td>M</td></tr>
 * <tr><td>0x0272</td><td>&#x0272</td><td>J</td><td>\textltailn</td></tr>
 * <tr><td>0x0273</td><td>&#x0273</td><td>n'</td><td>\:n</td></tr>
 * <tr><td>0x0274</td><td>&#x0274</td><td>N\</td><td>\;N</td></tr>
 * <tr><td>0x0275</td><td>&#x0275</td><td>8</td><td>8</td></tr>
 * <tr><td>0x0276</td><td>&#x0276</td><td>&amp;</td><td>\OE</td></tr>
 * <tr><td>0x0277</td><td>&#x0277</td><td></td><td>\textcloseomega</td></tr>
 * <tr><td>0x0278</td><td>&#x0278</td><td>p\</td><td>F</td></tr>
 * <tr><td>0x0279</td><td>&#x0279</td><td>r\</td><td>\*r</td></tr>
 * <tr><td>0x027a</td><td>&#x027a</td><td>l\</td><td>\textturnlonglegr</td></tr>
 * <tr><td>0x027b</td><td>&#x027b</td><td>r\'</td><td>\:R</td></tr>
 * <tr><td>0x027c</td><td>&#x027c</td><td></td><td>\textlonglegr</td></tr>
 * <tr><td>0x027d</td><td>&#x027d</td><td>r'</td><td>\:r</td></tr>
 * <tr><td>0x027e</td><td>&#x027e</td><td>4</td><td>R</td></tr>
 * <tr><td>0x0280</td><td>&#x0280</td><td>R</td><td>\;R</td></tr>
 * <tr><td>0x0281</td><td>&#x0281</td><td>R\</td><td>K</td></tr>
 * <tr><td>0x0282</td><td>&#x0282</td><td>s'</td><td>\:s</td></tr>
 * <tr><td>0x0283</td><td>&#x0283</td><td>S</td><td>S</td></tr>
 * <tr><td>0x0284</td><td>&#x0284</td><td>J\_&lt;</td><td>\!j</td></tr>
 * <tr><td>0x0286</td><td>&#x0286</td><td>S\</td><td>\textctesh</td></tr>
 * <tr><td>0x0287</td><td>&#x0287</td><td>t\</td><td>\*t</td></tr>
 * <tr><td>0x0288</td><td>&#x0288</td><td>t'</td><td>\:t</td></tr>
 * <tr><td>0x0289</td><td>&#x0289</td><td>}</td><td>0</td></tr>
 * <tr><td>0x028a</td><td>&#x028a</td><td>U</td><td>U</td></tr>
 * <tr><td>0x028b</td><td>&#x028b</td><td>P v\</td><td>V</td></tr>
 * <tr><td>0x028c</td><td>&#x028c</td><td>V</td><td>2</td></tr>
 * <tr><td>0x028d</td><td>&#x028d</td><td>W</td><td>\*w</td></tr>
 * <tr><td>0x028e</td><td>&#x028e</td><td>L</td><td>L</td></tr>
 * <tr><td>0x028f</td><td>&#x028f</td><td>Y</td><td>Y</td></tr>
 * <tr><td>0x0290</td><td>&#x0290</td><td>z'</td><td>\:z</td></tr>
 * <tr><td>0x0291</td><td>&#x0291</td><td>z\</td><td>\textctz</td></tr>
 * <tr><td>0x0292</td><td>&#x0292</td><td>Z</td><td>Z</td></tr>
 * <tr><td>0x0293</td><td>&#x0293</td><td></td><td>\textctyogh</td></tr>
 * <tr><td>0x0294</td><td>&#x0294</td><td>?</td><td>P</td></tr>
 * <tr><td>0x0295</td><td>&#x0295</td><td>?\</td><td>Q</td></tr>
 * <tr><td>0x0296</td><td>&#x0296</td><td></td><td>\textinvglotstop</td></tr>
 * <tr><td>0x0297</td><td>&#x0297</td><td>C\</td><td>\textstretchc</td></tr>
 * <tr><td>0x0298</td><td>&#x0298</td><td>O\</td><td>\!o</td></tr>
 * <tr><td>0x0299</td><td>&#x0299</td><td>B\</td><td>\;B</td></tr>
 * <tr><td>0x029a</td><td>&#x029a</td><td>E\</td><td>\textcloseepsilon</td></tr>
 * <tr><td>0x029b</td><td>&#x029b</td><td>G\_&lt;</td><td>\!G</td></tr>
 * <tr><td>0x029c</td><td>&#x029c</td><td>H\</td><td>\;H</td></tr>
 * <tr><td>0x029d</td><td>&#x029d</td><td>j\</td><td>J</td></tr>
 * <tr><td>0x029e</td><td>&#x029e</td><td>k\</td><td>\*k</td></tr>
 * <tr><td>0x029f</td><td>&#x029f</td><td>L\</td><td>\;L</td></tr>
 * <tr><td>0x02a0</td><td>&#x02a0</td><td></td><td>\texthtq</td></tr>
 * <tr><td>0x02a1</td><td>&#x02a1</td><td>&gt;\</td><td>\textbarglotstop</td></tr>
 * <tr><td>0x02a2</td><td>&#x02a2</td><td>&lt;\</td><td>\textbarrevglotstop</td></tr>
 * <tr><td>0x02a3</td><td>&#x02a3</td><td>dz</td><td>\textdzlig</td></tr>
 * <tr><td>0x02a4</td><td>&#x02a4</td><td>dZ</td><td>\textdyoghlig</td></tr>
 * <tr><td>0x02a5</td><td>&#x02a5</td><td>dz\</td><td>\textdctzlig</td></tr>
 * <tr><td>0x02a6</td><td>&#x02a6</td><td>ts</td><td>\texttslig</td></tr>
 * <tr><td>0x02a7</td><td>&#x02a7</td><td>tS</td><td>\textteshlig</td></tr>
 * <tr><td>0x02a8</td><td>&#x02a8</td><td>ts\</td><td>\texttctclig</td></tr>
 * <tr><td>0x02a9</td><td>&#x02a9</td><td>fN</td><td></td></tr>
 * <tr><td>0x02aa</td><td>&#x02aa</td><td>ls</td><td></td></tr>
 * <tr><td>0x02ab</td><td>&#x02ab</td><td>lz</td><td></td></tr>
 * <tr><td>0x02b0</td><td>&#x02b0</td><td>_h</td><td>\super{h}</td></tr>
 * <tr><td>0x02b1</td><td>&#x02b1</td><td>_h\</td><td>\super{H}</td></tr>
 * <tr><td>0x02b2</td><td>&#x02b2</td><td>_j</td><td>\super{j}</td></tr>
 * <tr><td>0x02b3</td><td>&#x02b3</td><td></td><td>\super{r}</td></tr>
 * <tr><td>0x02b4</td><td>&#x02b4</td><td>_r\</td><td>\super{*r}</td></tr>
 * <tr><td>0x02b5</td><td>&#x02b5</td><td>_r\'</td><td>\super{:R}</td></tr>
 * <tr><td>0x02b6</td><td>&#x02b6</td><td>_R\</td><td>\super{K}</td></tr>
 * <tr><td>0x02b7</td><td>&#x02b7</td><td>_w</td><td>\super{w}</td></tr>
 * <tr><td>0x02b8</td><td>&#x02b8</td><td>_y</td><td>\super{y}</td></tr>
 * <tr><td>0x02bc</td><td>&#x02bc</td><td>_&gt;</td><td>'</td></tr>
 * <tr><td>0x02c0</td><td>&#x02c0</td><td></td><td>\textraiseglotstop</td></tr>
 * <tr><td>0x02c1</td><td>&#x02c1</td><td></td><td>\super{Q}</td></tr>
 * <tr><td>0x02c8</td><td>&#x02c8</td><td>"</td><td>\textprimstress</td></tr>
 * <tr><td>0x02cc</td><td>&#x02cc</td><td>%</td><td>\textsecstress</td></tr>
 * <tr><td>0x02d0</td><td>&#x02d0</td><td>:</td><td>:</td></tr>
 * <tr><td>0x02d1</td><td>&#x02d1</td><td>:\</td><td>;</td></tr>
 * <tr><td>0x02d8</td><td>&#x02d8</td><td>-\</td><td></td></tr>
 * <tr><td>0x02de</td><td>&#x02de</td><td># `</td><td>\textrhoticity</td></tr>
 * <tr><td>0x02e0</td><td>&#x02e0</td><td>_G</td><td>\super{G}</td></tr>
 * <tr><td>0x02e1</td><td>&#x02e1</td><td>_l</td><td>\super{l}</td></tr>
 * <tr><td>0x02e2</td><td>&#x02e2</td><td>_s</td><td>\super{s}</td></tr>
 * <tr><td>0x02e3</td><td>&#x02e3</td><td></td><td>\super{x}</td></tr>
 * <tr><td>0x02e4</td><td>&#x02e4</td><td>_?\</td><td>\super{Q}</td></tr>
 * <tr><td>0x02e5</td><td>&#x02e5</td><td>&lt;T&gt;</td><td>\tone{55}</td></tr>
 * <tr><td>0x02e6</td><td>&#x02e6</td><td>&lt;H&gt;</td><td>\tone{44}</td></tr>
 * <tr><td>0x02e7</td><td>&#x02e7</td><td>&lt;M&gt;</td><td>\tone{33}</td></tr>
 * <tr><td>0x02e8</td><td>&#x02e8</td><td>&lt;L&gt;</td><td>\tone{22}</td></tr>
 * <tr><td>0x02e9</td><td>&#x02e9</td><td>&lt;B&gt;</td><td>\tone{11}</td></tr>
 * <tr><td>0x0300</td><td>&#x0300</td><td>_L</td><td>\`</td></tr>
 * <tr><td>0x0301</td><td>&#x0301</td><td>_H</td><td>\'</td></tr>
 * <tr><td>0x0302</td><td>&#x0302</td><td>_F _H_L _\</td><td>\^</td></tr>
 * <tr><td>0x0303</td><td>&#x0303</td><td>~</td><td>\~</td></tr>
 * <tr><td>0x0304</td><td>&#x0304</td><td>_M</td><td>\=</td></tr>
 * <tr><td>0x0306</td><td>&#x0306</td><td>_X</td><td>\\u</td></tr>
 * <tr><td>0x0308</td><td>&#x0308</td><td>_"</td><td>\"</td></tr>
 * <tr><td>0x030a</td><td>&#x030a</td><td></td><td>\r</td></tr>
 * <tr><td>0x030b</td><td>&#x030b</td><td>_T</td><td>\H</td></tr>
 * <tr><td>0x030c</td><td>&#x030c</td><td>_/ _L_H _R</td><td>\v</td></tr>
 * <tr><td>0x030f</td><td>&#x030f</td><td>_B</td><td>\H*</td></tr>
 * <tr><td>0x0318</td><td>&#x0318</td><td>_A</td><td>\|&lt;</td></tr>
 * <tr><td>0x0319</td><td>&#x0319</td><td>_q</td><td>\|&gt;</td></tr>
 * <tr><td>0x031a</td><td>&#x031a</td><td>_}</td><td>\textcorner</td></tr>
 * <tr><td>0x031c</td><td>&#x031c</td><td>_c</td><td>\|(</td></tr>
 * <tr><td>0x031d</td><td>&#x031d</td><td>_r</td><td>\|'</td></tr>
 * <tr><td>0x031e</td><td>&#x031e</td><td>_o</td><td>\|`</td></tr>
 * <tr><td>0x031f</td><td>&#x031f</td><td>_+</td><td>\|+</td></tr>
 * <tr><td>0x0320</td><td>&#x0320</td><td>_-</td><td>\=*</td></tr>
 * <tr><td>0x0321</td><td>&#x0321</td><td></td><td>\textpalhook</td></tr>
 * <tr><td>0x0322</td><td>&#x0322</td><td>'</td><td>\textrthook</td></tr>
 * <tr><td>0x0323</td><td>&#x0323</td><td>.</td><td>.</td></tr>
 * <tr><td>0x0324</td><td>&#x0324</td><td>_t</td><td>\"*</td></tr>
 * <tr><td>0x0325</td><td>&#x0325</td><td>_0</td><td>\r*</td></tr>
 * <tr><td>0x0329</td><td>&#x0329</td><td>=</td><td>\s</td></tr>
 * <tr><td>0x032a</td><td>&#x032a</td><td>_d</td><td>\|[</td></tr>
 * <tr><td>0x032b</td><td>&#x032b</td><td></td><td>\|w</td></tr>
 * <tr><td>0x032c</td><td>&#x032c</td><td>_v</td><td>\v*</td></tr>
 * <tr><td>0x032f</td><td>&#x032f</td><td>_^</td><td>\textsubarch</td></tr>
 * <tr><td>0x0330</td><td>&#x0330</td><td>_k</td><td>\~*</td></tr>
 * <tr><td>0x0334</td><td>&#x0334</td><td>_e</td><td>\|~</td></tr>
 * <tr><td>0x0339</td><td>&#x0339</td><td>_O</td><td>\|)</td></tr>
 * <tr><td>0x033a</td><td>&#x033a</td><td>_a</td><td>\|]</td></tr>
 * <tr><td>0x033b</td><td>&#x033b</td><td>_m</td><td>\textsubsquare</td></tr>
 * <tr><td>0x033c</td><td>&#x033c</td><td>_N</td><td>\|m</td></tr>
 * <tr><td>0x033d</td><td>&#x033d</td><td>_x</td><td>\|x</td></tr>
 * <tr><td>0x035d</td><td>&#x035c</td><td>_)</td><td>\t*{}</td></tr>
 * <tr><td>0x0361</td><td>&#x0361</td><td>_(</td><td>\t{}</td></tr>
 * <tr><td>0x03b2</td><td>&#x03b2</td><td>B</td><td>B</td></tr>
 * <tr><td>0x03b8</td><td>&#x03b8</td><td>T</td><td>T</td></tr>
 * <tr><td>0x03c7</td><td>&#x03c7</td><td>X</td><td>X</td></tr>
 * <tr><td>0x207f</td><td>&#x207f</td><td>_n</td><td>\super{n}</td></tr>
 * <tr><td>0x2191</td><td>&#x2191</td><td>^ &lt;^&gt;</td><td>\textupstep</td></tr>
 * <tr><td>0x2193</td><td>&#x2193</td><td>! &lt;!&gt;</td><td>\textdownstep</td></tr>
 * <tr><td>0x2197</td><td>&#x2197</td><td>&lt;/&gt; &lt;R&gt;</td><td>\textglobrise</td></tr>
 * <tr><td>0x2198</td><td>&#x2198</td><td>&lt;\&gt; &lt;F&gt;</td><td>\textglobfall</td></tr>
 * </table>

 */

public class IPA {
    /**
     * the IPA-String is just a unicode-String
     */
    private String m_ipa;

    /**
     * Create an empty IPA-String
     */
    public IPA() {
	m_ipa=null;
    }
    /**
     * Create and initialize an IPA-String.
     * The argument is (meant to be) a unicode-String holding
     * something of phonetic importance (pronunciation)
     * since unicode has all(?) the IPA-symbols, not much is done here
     * @param ipa duplicate the given IPA
     */
    public IPA(String ipa){
	m_ipa=new String(ipa);
    }
    /**
     * Create and initialize an IPA-String from SAMPA.
     * The argument is (meant to be) a String holding
     * something of phonetic importance (pronunciation) between
     * the delimiters s_start/s_stop.
     * everything between the delimiters is encoded from SAMPA to IPA/unicode
     * @param sampa the (partly) SAMPA-encoded transcription
     * @param s_start the starting delimiter of the SAMPA-encoded part
     * @param s_stop the stopping delimiter of the SAMPA-encoded part
     */
    public IPA(String sampa, String s_start, String s_stop){
	m_ipa=new String(ipa2sampaX(sampa, s_start, s_stop));
    }
    /**
     * Create and initialize an IPA-String from SAMPA.
     * Code that is enclosed by the delimiter, is converted from SAMPA to IPA/unicode
     * @param sampa the (partly) SAMPA-encoded transcription
     * @param s_delimiter the delimiter of the SAMPA-encoded part
     */
    public IPA(String sampa, String s_delimiter){
	m_ipa=new String(ipa2sampaX(sampa, s_delimiter, s_delimiter));
    }
    /**
     * Create and initialize an IPA-String from an array of bytes.
     * The byte-array is NOT the one, you would get when using String.toBytes()
     * because this one encodes the unicode-characters to locale (8bit)
     * most IPA-symbols would be gone then
     * instead it takes the bytes returned from IPA.toBytes() (16bit)
     * @param b_ipa the byte-encoded IPA-String
     */
    public IPA(byte[] b_ipa){
	char[] c_ipa = new char[b_ipa.length/2];
	int i=0, j=0;
	while (i<c_ipa.length){
	    /*
	    c_ipa[i]=(char)(((int)b_ipa[j])<<8+
			    ((b_ipa[j+1]<0)?256+(int)b_ipa[j+1]:(int)b_ipa[j+1]));
	    */
	    int i1=((int)b_ipa[j])<<8;
	    int i2=((b_ipa[j+1]<0)?256+(int)b_ipa[j+1]:(int)b_ipa[j+1]);
	    c_ipa[i]=(char)(i1+i2);
	    i++;   j+=2;
	    
	}
	m_ipa = new String(c_ipa);

	//	System.out.println("at.iem.tools.IPA: "+m_ipa);
    }
    /**
     * Set the IPA-String to the (IPA/unicode) argument
     * @param ipa the IPA-String to be used
     */
    public void setIPA(String ipa){
	m_ipa=new String(ipa);
    }
    /**
     * Set the IPA-String to the SAMPA-encoded argument
     * @param sampa the SAMPA-String to be used
     */
    public void setSAMPA(String sampa){
	m_ipa=new String(sampa2ipa(sampa));
    }
    /**
     * Set the IPA-String to the SAMPA-encoded argument.
     * Only the parts of the String that are enclosed by the s_start/s_stop delimiters,
     * will be decoded to IPA/unicode
     * @param sampa the (partly) SAMPA-String to be used
     * @param s_start the starting delimiter of the SAMPA-encoded part
     * @param s_stop the stopping delimiter of the SAMPA-encoded part
     */
    public void setSAMPA(String sampa, String s_start, String s_stop){
	m_ipa=new String(sampa2ipaX(sampa, s_start, s_stop));
    }
    /**
     * Set the IPA-String to the SAMPA-encoded argument.
     * Only the parts of the String that are enclosed by a pair of s_delimiter,
     * will be decoded to IPA/unicode
     * @param sampa the (partly) SAMPA-String to be used
     * @param s_delimiter the delimiter of the SAMPA-encoded part
     */
    public void setSAMPA(String sampa, String s_delimiter){
	m_ipa=new String(ipa2sampaX(sampa, s_delimiter, s_delimiter));
    }

    /**
     * Return a String-representation of the IPA-object
     */
    public String toString(){
	return m_ipa;
    }

    /**
     * prepend starting and append stopping delimiters if they are missing
     * @param s_start the starting delimiter
     * @param s_stop the stopping delimiter
     */
    public void  fixDelimiters(String s_start, String s_stop) {
        m_ipa=fixDelimiters(m_ipa, s_start, s_stop);
    }
    static String fixDelimiters(String ipa, String s_start, String s_stop) {
        // LATER rethink this
        int start0=ipa.indexOf(s_start);
        int start1=ipa.lastIndexOf(s_start);
        int stop0=ipa.indexOf(s_stop);
        int stop1=ipa.lastIndexOf(s_stop);

        if(-1==start0 && -1==stop0) {
            ipa=new String(s_start+ipa+s_stop);
        }
        else if(-1==start0) {
            ipa=new String(s_start+ipa);
        }
        else if(-1==stop0) {
            ipa=new String(ipa+s_stop);
        }
        else {
            if(start0>stop0) {
                ipa=new String(s_start+ipa);
            }
            if(stop1<start1) {
                ipa=new String(ipa+s_stop);
            }
        }

        return ipa;
    }

    /**
     * Return a String-representation of the IPA-object with fixed MS unicode bugs
     */
    public String toStringMS(){
        return fixUnicode2MS(m_ipa);
    }

    /**
     * Return a String-representation of the IPA-object; optionally fix the MS unicode bugs
     */
    public String toStringMS(boolean doFix){
	if(doFix)
            return toStringMS();
        return toString();
    }

    /**
     * Return a byte[]-representation of the IPA-object.
     * This is NOT the same, as String.getBytes().
     * The unicode-characters are encoded into a 16bit representation
     * and not into 8bit with the current locale-settings
     * @return a byte[]-array that holds the decoded IPA-String
     */
    public byte[] getBytes(){
	char[] c_ipa=m_ipa.toCharArray();
	byte[] b_ipa = new byte[c_ipa.length*2];
	int i=0, j=0;
	while(i<c_ipa.length){
	    b_ipa[j]=(byte)(c_ipa[i]>>8);
	    b_ipa[j+1]=(byte)(c_ipa[i]);
	    j+=2;
	    i++;
	}	
	return b_ipa;
    }

    /**
     * Return a byte[]-representation of the IPA-object.
     * you can specify an encodig, like UTF-8.
     * if the encoding is not supported by your system, <em>null</em> is returned
     * @param enc  the encoding that should be used
     * @return a byte[]-array that holds the decoded IPA-String
     */
    public byte[] getBytes(String enc){
	try{
	    return m_ipa.getBytes(enc);
	} catch (Exception e){
	    return null;
	}
    }

    /**
     * get the SAMPA-representation of the IPA-object
     * @return the SAMPA-encoded transcription String
     */
    public String getSAMPA(){
	return ipa2sampa(m_ipa);
    }
    /**
     * Encode the part of the IPA-object 
     * that is enclosed by the s_start/s_stop delimiters
     * into SAMPA, leave the rest unchanged
     * @param s_start the starting-delimiter of the transcription
     * @param s_stop the stopping-delimiter of the transcription
     * @return the partly SAMPA-encoded transcription String
     */
    public String getSAMPA(String s_start, String s_stop){
	return ipa2sampaX(m_ipa, s_start, s_stop);
    }
    /**
     * Encode the part of the IPA-object 
     * that is enclosed by the s_delimiter
     * into SAMPA, leave the rest unchanged
     * @param s_delimiter the start/stop delimiter
     * @return the partly SAMPA-encoded transcription String
     */
    public String getSAMPA(String s_delimiter){
	return ipa2sampaX(m_ipa, s_delimiter, s_delimiter);
    }
    /**
     * Encode the IPA-object into TIPA
     * @return the TIPA-encoded transcription String
     */
    public String getTIPA(){
	return ipa2tipa(m_ipa);
    }
    /**
     * Encode the part of the IPA-object 
     * that is enclosed by the s_start/stop delimiters
     * into TIPA, leave the rest unchanged
     * @param s_start the starting-delimiter of the transcription
     * @param s_stop the stopping-delimiter of the transcription
     * @return the partly TIPA-encoded transcription String
     */
    public String getTIPA(String s_start, String s_stop){
	return ipa2tipa(m_ipa, s_start, s_stop);
    }
    /**
     * Encode the part of the IPA-object 
     * that is enclosed by the s_delimiter
     * into TIPA, leave the rest unchanged
     * @param s_delimiter the start/stop delimiter
     * @return the partly TIPA-encoded transcription String
    */
    public String getTIPA(String s_delimiter){
	return ipa2tipa(m_ipa, s_delimiter, s_delimiter);
    }

    // Here come the static functions


    /**
     * convert a given String vom SAMPA to IPA
     * @param sampa the String encoded in SAMPA
     * @return the String encoded in IPA
     */
    public static String sampa2ipa(String sampa){
	String ipa = sampa;
	ipa = replace(ipa, "|\\|\\", "\u01c1");
	ipa = replace(ipa, "_R_F", "\u0000");
	ipa = replace(ipa, "_L_H", "\u030c");
	ipa = replace(ipa, "_H_T", "\u0000");
	ipa = replace(ipa, "_H_L", "\u0302");
	ipa = replace(ipa, "_B_T", "\u0000");
	ipa = replace(ipa, "_r\\'", "\u02b5");
	ipa = replace(ipa, "J\\_<", "\u0284");
	ipa = replace(ipa, "G\\_<", "\u029b");
	ipa = replace(ipa, "ts\\", "\u02a8");
	ipa = replace(ipa, "r\\'", "\u027b");
	ipa = replace(ipa, "l_e", "\u026b");
	ipa = replace(ipa, "g_<", "\u0260");
	ipa = replace(ipa, "dz\\", "\u02a5");
	ipa = replace(ipa, "d_<", "\u0257");
	ipa = replace(ipa, "b_<", "\u0253");
	ipa = replace(ipa, "_R\\", "\u02b6");
	ipa = replace(ipa, "_r\\", "\u02b4");
	ipa = replace(ipa, "_r'", "\u02b3");  // not really SAMPA
	ipa = replace(ipa, "_h\\", "\u02b1");
	ipa = replace(ipa, "_?\\", "\u02e4");
	ipa = replace(ipa, "_?'", "\u02c1"); // not really SAMPA
	ipa = replace(ipa, "_x'", "\u02e3"); // not really SAMPA
	ipa = replace(ipa, "<^>", "\u2191");
	ipa = replace(ipa, "<\\>", "\u2198");
	ipa = replace(ipa, "<T>", "\u02e5");
	ipa = replace(ipa, "<R>", "\u2197");
	ipa = replace(ipa, "<M>", "\u02e7");
	ipa = replace(ipa, "<L>", "\u02e8");
	ipa = replace(ipa, "<H>", "\u02e6");
	ipa = replace(ipa, "<F>", "\u2198");
	ipa = replace(ipa, "<B>", "\u02e9");
	ipa = replace(ipa, "</>", "\u2197");
	ipa = replace(ipa, "<!>", "\u2193");
	ipa = replace(ipa, "|\\", "\u01c0");
	ipa = replace(ipa, "z\\", "\u0291");
	ipa = replace(ipa, "z'", "\u0290");
	ipa = replace(ipa, "x\\", "\u0267");
	ipa = replace(ipa, "v\\", "\u028b");
	ipa = replace(ipa, "ts", "\u02a6");
	ipa = replace(ipa, "tS", "\u02a7");
	ipa = replace(ipa, "t\\", "\u0287");
	ipa = replace(ipa, "t'", "\u0288");
	ipa = replace(ipa, "s\\", "\u0255");
	ipa = replace(ipa, "s'", "\u0282");
	ipa = replace(ipa, "r\\", "\u0279");
	ipa = replace(ipa, "r'", "\u027d");
	ipa = replace(ipa, "p\\", "\u0278");
	ipa = replace(ipa, "q\\", "\u02a0"); // not really SAMPA
	ipa = replace(ipa, "n\\", "\u019e");
	ipa = replace(ipa, "n'", "\u0273");
	//	ipa = replace(ipa, "lz", "\u02ab"); // ? some fonts don't know this 
	//	ipa = replace(ipa, "ls", "\u02aa"); // ?   -"-
	ipa = replace(ipa, "l\\", "\u027a");
	ipa = replace(ipa, "l'", "\u026d");
	ipa = replace(ipa, "k\\", "\u029e");
	ipa = replace(ipa, "j\\", "\u029d");
	ipa = replace(ipa, "i\\", "\u0269");
	ipa = replace(ipa, "h\\", "\u0266");
	//	ipa = replace(ipa, "fN", "\u02a9"); // ?  -"-
	ipa = replace(ipa, "dz", "\u02a3");
	ipa = replace(ipa, "dZ", "\u02a4");
	ipa = replace(ipa, "d'", "\u0256");
	ipa = replace(ipa, "_}", "\u031a");
	ipa = replace(ipa, "_y", "\u02b8");
	ipa = replace(ipa, "_x", "\u033d");
	ipa = replace(ipa, "_w", "\u02b7");
	ipa = replace(ipa, "_v", "\u032c");
	ipa = replace(ipa, "_t", "\u0324");
	ipa = replace(ipa, "_s", "\u02e2");
	ipa = replace(ipa, "_r", "\u031d");
	ipa = replace(ipa, "_q", "\u0319");
	ipa = replace(ipa, "_o", "\u031e");
	ipa = replace(ipa, "_m", "\u033b");
	ipa = replace(ipa, "_l", "\u02e1");
	ipa = replace(ipa, "_k", "\u0330");
	ipa = replace(ipa, "_j", "\u02b2");
	ipa = replace(ipa, "_h", "\u02b0");
	ipa = replace(ipa, "_n", "\u207f");
	ipa = replace(ipa, "_e", "\u0334");
	ipa = replace(ipa, "_d", "\u032a");
	ipa = replace(ipa, "_c", "\u031c");
	ipa = replace(ipa, "_a", "\u033a");
	ipa = replace(ipa, "_^", "\u032f");
	ipa = replace(ipa, "_\\", "\u0302");
	ipa = replace(ipa, "_?", "\u02c0"); // not really SAMPA
	ipa = replace(ipa, "_X", "\u0306");
	ipa = replace(ipa, "_T", "\u030b");
	ipa = replace(ipa, "_R", "\u030c");
	ipa = replace(ipa, "_O", "\u0339");
	ipa = replace(ipa, "_N", "\u033c");
	ipa = replace(ipa, "_M", "\u0304");
	ipa = replace(ipa, "_L", "\u0300");
	ipa = replace(ipa, "_H", "\u0301");
	ipa = replace(ipa, "_G", "\u02e0");
	ipa = replace(ipa, "_F", "\u0302");
	ipa = replace(ipa, "_B", "\u030f");
	ipa = replace(ipa, "_A", "\u0318");
	ipa = replace(ipa, "_>", "\u02bc");
	ipa = replace(ipa, "_0", "\u0325");
	ipa = replace(ipa, "_/", "\u030c");
	ipa = replace(ipa, "_-", "\u0320");
	ipa = replace(ipa, "_+", "\u031f");
	ipa = replace(ipa, "_(", "\u0361");
	ipa = replace(ipa, "_)", "\u035c");
	ipa = replace(ipa, "_\"", "\u0308");
	ipa = replace(ipa, "Z\\", "\u01b9");
	ipa = replace(ipa, "X\\", "\u0127");
	ipa = replace(ipa, "S\\", "\u0286");
	ipa = replace(ipa, "R\\", "\u0281");
	ipa = replace(ipa, "O\\", "\u0298");
	ipa = replace(ipa, "N\\", "\u0274");
	ipa = replace(ipa, "M\\", "\u0270");
	ipa = replace(ipa, "L\\", "\u029f");
	ipa = replace(ipa, "K\\", "\u026e");
	ipa = replace(ipa, "J\\", "\u025f");
	ipa = replace(ipa, "H\\", "\u029c");
	ipa = replace(ipa, "G\\", "\u0262");
	ipa = replace(ipa, "E\\", "\u029a");
	ipa = replace(ipa, "C\\", "\u0297");
	ipa = replace(ipa, "B\\", "\u0299");
	ipa = replace(ipa, "@`", "\u025a");
	ipa = replace(ipa, "@\\", "\u0258");
	ipa = replace(ipa, "?\\", "\u0295");
	ipa = replace(ipa, ">\\", "\u02a1");
	ipa = replace(ipa, "=\\", "\u01c2");
	ipa = replace(ipa, "<\\", "\u02a2");
	ipa = replace(ipa, ":\\", "\u02d1");
	ipa = replace(ipa, "3\\", "\u025e");
	ipa = replace(ipa, "-\\", "\u02d8");
	ipa = replace(ipa, "!\\", "\u01c3");
	ipa = replace(ipa, "~", "\u0303");
	ipa = replace(ipa, "}", "\u0289");
	ipa = replace(ipa, "{", "\u00e6");
	ipa = replace(ipa, "g", "\u0261");
	ipa = replace(ipa, "\u0261\\", "g");
	ipa = replace(ipa, "`", "\u02de");
	ipa = replace(ipa, "^", "\u2191");
	ipa = replace(ipa, "Z", "\u0292");
	ipa = replace(ipa, "Y", "\u028f");
	ipa = replace(ipa, "X", "\u03c7");
	ipa = replace(ipa, "W", "\u028d");
	ipa = replace(ipa, "V", "\u028c");
	ipa = replace(ipa, "U", "\u028a");
	ipa = replace(ipa, "T", "\u03b8");
	ipa = replace(ipa, "S", "\u0283");
	ipa = replace(ipa, "R", "\u0280");
	ipa = replace(ipa, "Q", "\u0252");
	ipa = replace(ipa, "P", "\u028b");
	ipa = replace(ipa, "O", "\u0254");
	ipa = replace(ipa, "N", "\u014b");
	ipa = replace(ipa, "M", "\u026f");
	ipa = replace(ipa, "L", "\u028e");
	ipa = replace(ipa, "K", "\u026c");
	ipa = replace(ipa, "J", "\u0272");
	ipa = replace(ipa, "I", "\u026a");
	ipa = replace(ipa, "H", "\u0265");
	ipa = replace(ipa, "G", "\u0263");
	ipa = replace(ipa, "F", "\u0271");
	ipa = replace(ipa, "E", "\u025b");
	ipa = replace(ipa, "D", "\u00f0");
	ipa = replace(ipa, "C", "\u00e7");
	ipa = replace(ipa, "B", "\u03b2");
	ipa = replace(ipa, "A", "\u0251");
	ipa = replace(ipa, "@", "\u0259");
	ipa = replace(ipa, "?", "\u0294");
	ipa = replace(ipa, "=", "\u0329");
	ipa = replace(ipa, ":", "\u02d0");
	ipa = replace(ipa, "9", "\u0153");
	ipa = replace(ipa, "8", "\u0275");
	ipa = replace(ipa, "7", "\u0264");
	ipa = replace(ipa, "6", "\u0250");
	ipa = replace(ipa, "5", "\u026b");
	ipa = replace(ipa, "4", "\u027e");
	ipa = replace(ipa, "3", "\u025c");
	ipa = replace(ipa, "2", "\u00f8");
	ipa = replace(ipa, "1", "\u0268");
	ipa = replace(ipa, "'", "\u0322");
	ipa = replace(ipa, "&", "\u0276");
	ipa = replace(ipa, "%", "\u02cc");
	ipa = replace(ipa, "#", "\u02de");
	ipa = replace(ipa, "\"", "\u02c8");
	ipa = replace(ipa, "!", "\u2193");

	return ipa;
    }

    /**
     * convert a given String vom IPA to SAMPA
     * @param ipa the String encoded in IPA
     * @return the String encoded in SAMPA
     */
    public static String ipa2sampa(String ipa){
	String sampa = ipa;
	sampa = replace(sampa, "'", "\"");

	sampa = replace(sampa, "\u01c1", "|\\|\\");
	sampa = replace(sampa, "\u0000", "_R_F");
	sampa = replace(sampa, "\u030c", "_L_H");
	sampa = replace(sampa, "\u0000", "_H_T");
	sampa = replace(sampa, "\u0302", "_H_L");
	sampa = replace(sampa, "\u0000", "_B_T");
	sampa = replace(sampa, "\u02b5", "_r\\'");
	sampa = replace(sampa, "\u0284", "J\\_<");
	sampa = replace(sampa, "\u029b", "G\\_<");
	sampa = replace(sampa, "\u02a8", "ts\\");
	sampa = replace(sampa, "\u027b", "r\\'");
	sampa = replace(sampa, "\u026b", "l_e");
	sampa = replace(sampa, "\u0260", "g_<");
	sampa = replace(sampa, "\u02a5", "dz\\");
	sampa = replace(sampa, "\u0257", "d_<");
	sampa = replace(sampa, "\u0253", "b_<");
	sampa = replace(sampa, "\u02b6", "_R\\");
	sampa = replace(sampa, "\u02b4", "_r\\");
	sampa = replace(sampa, "\u02b1", "_h\\");
	sampa = replace(sampa, "\u02e4", "_?\\");
	sampa = replace(sampa, "\u2191", "<^>");
	sampa = replace(sampa, "\u2198", "<\\>");
	sampa = replace(sampa, "\u02e5", "<T>");
	sampa = replace(sampa, "\u2197", "<R>");
	sampa = replace(sampa, "\u02e7", "<M>");
	sampa = replace(sampa, "\u02e8", "<L>");
	sampa = replace(sampa, "\u02e6", "<H>");
	sampa = replace(sampa, "\u2198", "<F>");
	sampa = replace(sampa, "\u02e9", "<B>");
	sampa = replace(sampa, "\u2197", "</>");
	sampa = replace(sampa, "\u2193", "<!>");
	sampa = replace(sampa, "\u01c0", "|\\");
	sampa = replace(sampa, "\u0291", "z\\");
	sampa = replace(sampa, "\u0290", "z'");
	sampa = replace(sampa, "\u0267", "x\\");
	sampa = replace(sampa, "\u028b", "v\\");
	sampa = replace(sampa, "\u02a6", "ts");
	sampa = replace(sampa, "\u02a7", "tS");
	sampa = replace(sampa, "\u0287", "t\\");
	sampa = replace(sampa, "\u0288", "t'");
	sampa = replace(sampa, "\u0255", "s\\");
	sampa = replace(sampa, "\u0282", "s'");
	sampa = replace(sampa, "\u0279", "r\\");
	sampa = replace(sampa, "\u027d", "r'");
	sampa = replace(sampa, "\u0278", "p\\");
	sampa = replace(sampa, "\u019e", "n\\");
	sampa = replace(sampa, "\u0273", "n'");
	sampa = replace(sampa, "\u02ab", "lz");
	sampa = replace(sampa, "\u02aa", "ls");
	sampa = replace(sampa, "\u027a", "l\\");
	sampa = replace(sampa, "\u026d", "l'");
	sampa = replace(sampa, "\u029e", "k\\");
	sampa = replace(sampa, "\u029d", "j\\");
	sampa = replace(sampa, "\u0269", "i\\");
	sampa = replace(sampa, "\u0266", "h\\");
	sampa = replace(sampa, "\u02a9", "fN");
	sampa = replace(sampa, "\u02a3", "dz");
	sampa = replace(sampa, "\u02a4", "dZ");
	sampa = replace(sampa, "\u0256", "d'");
	sampa = replace(sampa, "\u031a", "_}");
	sampa = replace(sampa, "\u02b8", "_y");
	sampa = replace(sampa, "\u033d", "_x");
	sampa = replace(sampa, "\u02b7", "_w");
	sampa = replace(sampa, "\u032c", "_v");
	sampa = replace(sampa, "\u0324", "_t");
	sampa = replace(sampa, "\u02e2", "_s");
	sampa = replace(sampa, "\u031d", "_r");
	sampa = replace(sampa, "\u0319", "_q");
	sampa = replace(sampa, "\u031e", "_o");
	sampa = replace(sampa, "\u033b", "_m");
	sampa = replace(sampa, "\u02e1", "_l");
	sampa = replace(sampa, "\u0330", "_k");
	sampa = replace(sampa, "\u02b2", "_j");
	sampa = replace(sampa, "\u02b0", "_h");
	sampa = replace(sampa, "\u207f", "_n");
	sampa = replace(sampa, "\u0334", "_e");
	sampa = replace(sampa, "\u032a", "_d");
	sampa = replace(sampa, "\u031c", "_c");
	sampa = replace(sampa, "\u033a", "_a");
	sampa = replace(sampa, "\u032f", "_^");
	sampa = replace(sampa, "\u0302", "_\\");
	sampa = replace(sampa, "\u02c0", "_?"); // not really SAMPA
	sampa = replace(sampa, "\u0306", "_X");
	sampa = replace(sampa, "\u030b", "_T");
	sampa = replace(sampa, "\u030c", "_R");
	sampa = replace(sampa, "\u0339", "_O");
	sampa = replace(sampa, "\u033c", "_N");
	sampa = replace(sampa, "\u0304", "_M");
	sampa = replace(sampa, "\u0300", "_L");
	sampa = replace(sampa, "\u0301", "_H");
	sampa = replace(sampa, "\u02e0", "_G");
	sampa = replace(sampa, "\u0302", "_F");
	sampa = replace(sampa, "\u030f", "_B");
	sampa = replace(sampa, "\u0318", "_A");
	sampa = replace(sampa, "\u02bc", "_>");
	sampa = replace(sampa, "\u0325", "_0");
	sampa = replace(sampa, "\u030c", "_/");
	sampa = replace(sampa, "\u0320", "_-");
	sampa = replace(sampa, "\u031f", "_+");
	sampa = replace(sampa, "\u0361", "_(");
	sampa = replace(sampa, "\u035c", "_)");
	sampa = replace(sampa, "\u0308", "_\"");
	sampa = replace(sampa, "\u01b9", "Z\\");
	sampa = replace(sampa, "\u0127", "X\\");
	sampa = replace(sampa, "\u0286", "S\\");
	sampa = replace(sampa, "\u0281", "R\\");
	sampa = replace(sampa, "\u0298", "O\\");
	sampa = replace(sampa, "\u0274", "N\\");
	sampa = replace(sampa, "\u0270", "M\\");
	sampa = replace(sampa, "\u029f", "L\\");
	sampa = replace(sampa, "\u026e", "K\\");
	sampa = replace(sampa, "\u025f", "J\\");
	sampa = replace(sampa, "\u029c", "H\\");
	sampa = replace(sampa, "\u0262", "G\\");
	sampa = replace(sampa, "\u029a", "E\\");
	sampa = replace(sampa, "\u0297", "C\\");
	sampa = replace(sampa, "\u0299", "B\\");
	sampa = replace(sampa, "\u025a", "@`");
	sampa = replace(sampa, "\u0258", "@\\");
	sampa = replace(sampa, "\u0295", "?\\");
	sampa = replace(sampa, "\u02a1", ">\\");
	sampa = replace(sampa, "\u01c2", "=\\");
	sampa = replace(sampa, "\u02a2", "<\\");
	sampa = replace(sampa, "\u02d1", ":\\");
	sampa = replace(sampa, "\u025e", "3\\");
	sampa = replace(sampa, "\u02d8", "-\\");
	sampa = replace(sampa, "\u01c3", "!\\");
	sampa = replace(sampa, "\u0303", "~");
	sampa = replace(sampa, "\u0289", "}");
	sampa = replace(sampa, "\u00e6", "{");
	sampa = replace(sampa, "\u02a0", "q\\"); // not really SAMPA
	sampa = replace(sampa, "g",      "g\\");
	sampa = replace(sampa, "\u0261", "g");
	sampa = replace(sampa, "\u02de", "`");
	sampa = replace(sampa, "\u2191", "^");
	sampa = replace(sampa, "\u0292", "Z");
	sampa = replace(sampa, "\u028f", "Y");
	sampa = replace(sampa, "\u03c7", "X");
	sampa = replace(sampa, "\u028d", "W");
	sampa = replace(sampa, "\u028c", "V");
	sampa = replace(sampa, "\u028a", "U");
	sampa = replace(sampa, "\u03b8", "T");
	sampa = replace(sampa, "\u0283", "S");
	sampa = replace(sampa, "\u0280", "R");
	sampa = replace(sampa, "\u0252", "Q");
	sampa = replace(sampa, "\u028b", "P");
	sampa = replace(sampa, "\u0254", "O");
	sampa = replace(sampa, "\u014b", "N");
	sampa = replace(sampa, "\u026f", "M");
	sampa = replace(sampa, "\u028e", "L");
	sampa = replace(sampa, "\u026c", "K");
	sampa = replace(sampa, "\u0272", "J");
	sampa = replace(sampa, "\u026a", "I");
	sampa = replace(sampa, "\u0265", "H");
	sampa = replace(sampa, "\u0263", "G");
	sampa = replace(sampa, "\u0271", "F");
	sampa = replace(sampa, "\u025b", "E");
	sampa = replace(sampa, "\u00f0", "D");
	sampa = replace(sampa, "\u00e7", "C");
	sampa = replace(sampa, "\u03b2", "B");
	sampa = replace(sampa, "\u0251", "A");
	sampa = replace(sampa, "\u0259", "@");
	sampa = replace(sampa, "\u0294", "?");
	sampa = replace(sampa, "\u0329", "=");
	sampa = replace(sampa, "\u02d0", ":");
	sampa = replace(sampa, "\u0153", "9");
	sampa = replace(sampa, "\u0275", "8");
	sampa = replace(sampa, "\u0264", "7");
	sampa = replace(sampa, "\u0250", "6");
	sampa = replace(sampa, "\u026b", "5");
	sampa = replace(sampa, "\u027e", "4");
	sampa = replace(sampa, "\u025c", "3");
	sampa = replace(sampa, "\u00f8", "2");
	sampa = replace(sampa, "\u0268", "1");
	sampa = replace(sampa, "\u0322", "'");
	sampa = replace(sampa, "\u0276", "&");
	sampa = replace(sampa, "\u02cc", "%");
	sampa = replace(sampa, "\u02de", "#");
	sampa = replace(sampa, "\u02c8", "\"");
	sampa = replace(sampa, "\u2193", "!");

	sampa = replace(sampa, "\u02b3", "_r'"); //  not really SAMPA
	sampa = replace(sampa, "\u02c1", "_?'"); // not really SAMPA
	sampa = replace(sampa, "\u02e3", "_x'"); // not really SAMPA

	return sampa;
    }


    // phonetic groups

    public static final String[] DIACRITICS = {
	"\u0325", "\u032c", "\u02b1", "\u0339", "\u031c", "\u031f", "\u0320", "\u0308", "\u033d",
	"\u0329", "\u032f", "\u02de", "\u0324", "\u0330", "\u033c", "\u02b7", "\u02b2", "\u02e0",
	"\u02e4", "\u032a", "\u033a", "\u033b", "\u0303", "\u207f", "\u02e1", "\u031a", "\u0334",
	"\u031d", "\u031e", "\u0318", "\u0319", "\u02b5", "\u02b6", "\u02b4", "\u02b8", "\u02e2",
	"\u02b0", "\u02c0", "\u02b3", "\u02c1", "\u02e3", 
	
	"\u0322", // ' (rhotacity?)
	"\u02bc" // _> (ejective) does this belong here ???
    };
    public static final String[] TONES = {
	"\u0300", "\u0301", "\u0304", "\u030b", "\u030f", 
	"\u2191", "\u2198", "\u02e5", "\u2197", "\u02e7", "\u02e8", "\u02e6", "\u2198", "\u02e9",
	"\u2197", "\u2193", 
	"\u0302", "\u030c"
    };
    public static final String[] SUPRASEGMENTALS={
	"\u02c8", "\u02cc", "\u02d0", "\u02d1", "\u0306", "||", "|", ".", "\u02d8", 
	"\u0361"
    };
    public static final String[] VOWELS={
	"i", "e", "\u025b", "a", "\u0251", "\u0254", "o", "u", "y", 
	"\u00f8", "\u0153", "\u0276", "\u0252", "\u028c", "\u0264", "\u026f", 
	"\u0268", "\u0289", "\u026a", "\u028f", "\u028a", "\u0259", "\u0275", 
	"\u0250", "\u00e6", "\u025c", "\u025e", "\u0258", "\u025a", 
	"\u029a", "\u0269"
    };
    public static final String[] PLOSIVES={
	"p", "b", "t", "d", "\u0288", "\u0256", "c", "\u025f", "k", "g",
	"\u0261", "q", "\u0262", "\u0294", "\u02a1", "\u0287", "\u029e"
    };
    public static final String[] NASALS={
	"m", "\u0271", "n", "\u0273", "\u0272", "\u014b", "\u0274"
    };
    public static final String[] TRILLS={
	"\u0299", "r", "\u0280"
    };
    public static final String[] FLAPS={
	"\u027e", "\u027d",
	"\u027a"
    };
    public static final String[] FRICATIVES={
	"\u0278", "\u03b2", "f", "v", "\u03b8", "\u00f0", "s", "z", "\u0283", "\u0292", "\u0282",
	"\u0290", "\u00e7", "\u029d", "x", "\u0263", "\u03c7", "\u0281", "\u0127", "\u0295", "h", 
	"\u0266", 

	"\u028d", "\u029c", "\u02a2", 
	"\u0255", "\u0291", "\u0267", "\u0286"
    };
    public static final String[] LATERAL_FRICATIVES={
	"\u026c", "\u026e"
    };
    public static final String[] APPROXIMANTS={
	"\u028b", "\u0279", "\u027b", "j", "\u0270", 
	"w", "\u0265"
    };
    public static final String[] LATERAL_APPROXIMANTS={
	"l", 	"\u027a", "\u026d", "\u028e", "\u029f"
    };
    public static final String[] CLICKS={
	"\u0298", "\u01c0", "\u01c3", "\u01c2", "\u01c1"
    };

    public static final String[] IMPLOSIVES={
	"\u0284", "\u029b", "\u0260", "\u0257", "\u0253"
    };


    public static final String[] BILABIALS={
	"p", "b", "m", "\u0299", "\u0278", "\u03b2", "\u0298", "\u0284"
    };
    public static final String[] LABIODENTALS={
	"\u0271", "f", "v", "\u028b"	
    };
    public static final String[] DENTALS={
	"\u03b8", "\u00f0", "\u01c0", "\u029b"
    };
    public static final String[] ALVEOLARS={
	"t", "d", "n", "r", "\u027e", "s", "z", "\u026c", "\u026e","\u0279", "l", "\u01c3", "\u01c2", "\u01c1"
    };
    public static final String[] POSTALVEOLARS={
	"\u0283", "\u0292", "\u01c3"
    };
    public static final String[] RETROFLEXS={
	"\u0288", "\u0256", "\u0273", "\u027d", "\u0282", "\u0290", "\u027b", "\u027a"
    };
    public static final String[] PALATALS={
	"c", "\u025f", "\u0272", "\u00e7", "\u029d", "j", "\u026d", "\u0260"
    };
    public static final String[] VELARS={
	"k", "g",  "\u014b", "x", "\u0263", "\u0270", "\u029f", "\u0257"
    };
    public static final String[] UVULARS={
	"q", "\u0262", "\u0274", "\u0280", "\u03c7", "\u0281", "\u0253"
    };
    public static final String[] PHARYNGEALS={
	"\u0127", "\u0295"
    };
    public static final String[] GLOTTALS={
	"\u029e", "h", "\u0266"
    };
    public static final String[] EPIGLOTTALS={
	"\u0262", "\u02a1", "\u02a2"
    };

    public static final String[] EJECTIVES={
	"\u02bc", 
    };

    public static final String[] COMBINED={
	"\u028d", "w", "\u0265", "\u0255", "\u0291", "\u027a", "\u0267", 
    };

    private static final String[][]CONSONANT2={PLOSIVES, 
					       NASALS,
					       TRILLS, 
					       FLAPS,
					       FRICATIVES,
					       LATERAL_FRICATIVES,
					       APPROXIMANTS,
					       LATERAL_APPROXIMANTS};
    public static final String[] CONSONANTS=at.iem.tools.StringX.concatenate(CONSONANT2);

    /* unknown thingies...
     * consonants: 026b, 0297
     *
     */


    /**
     * replace all diacritics within the ipa-string with "diacritic"
     * @param ipa the String encoded in IPA
     * @param diacritic the string that replaces all diacritics
     * @return the String with all diacritics replaced
     */
    public static String stripDiacritics(String ipa, String diacritic){
	return at.iem.tools.StringX.replace(ipa, DIACRITICS, diacritic);
    }
    
    /**
     * replace all tones within the ipa-string with "tone"
     * @param ipa the String encoded in IPA
     * @param tone the string that replaces all tones
     * @return the String with all tones replaced
     */
    public static String stripTones(String ipa, String tone){
	return at.iem.tools.StringX.replace(ipa, TONES, tone);
    }
    
    /**
     * replace all suprasegmentals within the ipa-string with "suprasegmental"
     * @param ipa the String encoded in IPA
     * @param suprasegmental the string that replaces all suprasegmentals
     * @return the String with all suprasegmentals replaced
     */
    public static String stripSuprasegmentals(String ipa, String suprasegmental){
	return at.iem.tools.StringX.replace(ipa, SUPRASEGMENTALS, suprasegmental);
    }

    /**
     * replace all vowels within the ipa-string with "vowel"
     * @param ipa the String encoded in IPA
     * @param vowel the string that replaces all vowels
     * @return the String with all vowels replaced
     */
    public static String stripVowels(String ipa, String vowel){
	return at.iem.tools.StringX.replace(ipa, VOWELS, vowel);
    }

    /**
     * replace all plosives within the ipa-string with "plosive"
     * @param ipa the String encoded in IPA
     * @param plosive the string that replaces all plosives
     * @return the String with all plosives replaced
     */
    public static String stripPlosives(String ipa, String plosive){
	return at.iem.tools.StringX.replace(ipa, PLOSIVES, plosive);
    }

    /**
     * replace all nasals within the ipa-string with "nasal"
     * @param ipa the String encoded in IPA
     * @param nasal the string that replaces all nasals
     * @return the String with all nasals replaced
     */
    public static String stripNasals(String ipa, String nasal){
	return at.iem.tools.StringX.replace(ipa, NASALS, nasal);
    }

    /**
     * replace all trills within the ipa-string with "trill"
     * @param ipa the String encoded in IPA
     * @param trill the string that replaces all trills
     * @return the String with all trills replaced
     */
    public static String stripTrills(String ipa, String trill){
	return at.iem.tools.StringX.replace(ipa, TRILLS, trill);
    }

    /**
     * replace all flaps within the ipa-string with "flap"
     * @param ipa the String encoded in IPA
     * @param flap the string that replaces all flaps
     * @return the String with all flaps replaced
     */
    public static String stripFlaps(String ipa, String flap){
	return at.iem.tools.StringX.replace(ipa, FLAPS, flap);
    }

    /**
     * replace all fricatives within the ipa-string with "fricative"
     * @param ipa the String encoded in IPA
     * @param fricative the string that replaces all fricatives
     * @return the String with all fricatives replaced
     */
    public static String stripFricatives(String ipa, String fricative){
	return at.iem.tools.StringX.replace(ipa, FRICATIVES, fricative);
    }


    /**
     * replace all lateral fricatives within the ipa-string with "lfricative"
     * @param ipa the String encoded in IPA
     * @param lfricative the string that replaces all lfricatives
     * @return the String with all lfricatives replaced
     */
    public static String stripLFricatives(String ipa, String lfricative){
	return at.iem.tools.StringX.replace(ipa, LATERAL_FRICATIVES, lfricative);
    }

    /**
     * replace all approximants within the ipa-string with "approximant"
     * @param ipa the String encoded in IPA
     * @param approximant the string that replaces all approximants
     * @return the String with all approximants replaced
     */
    public static String stripApproximants(String ipa, String approximant){
	return at.iem.tools.StringX.replace(ipa, APPROXIMANTS, approximant);
    }

    /**
     * replace all lateral approximants within the ipa-string with "lapproximant"
     * @param ipa the String encoded in IPA
     * @param lapproximant the string that replaces all lapproximants
     * @return the String with all lapproximants replaced
     */
    public static String stripLApproximants(String ipa, String lapproximant){
	return at.iem.tools.StringX.replace(ipa, LATERAL_APPROXIMANTS, lapproximant);
    }

    /**
     * replace all clicks within the ipa-string with "click"
     * @param ipa the String encoded in IPA
     * @param click the string that replaces all clicks
     * @return the String with all clicks replaced
     */
    public static String stripClicks(String ipa, String click){
	return at.iem.tools.StringX.replace(ipa, CLICKS, click);
    }

    /**
     * replace all implosives within the ipa-string with "implosive"
     * @param ipa the String encoded in IPA
     * @param implosive the string that replaces all implosives
     * @return the String with all implosives replaced
     */
    public static String stripImplosives(String ipa, String implosive){
	return at.iem.tools.StringX.replace(ipa, IMPLOSIVES, implosive);
    }


    public String stripDiacritics(String diacritic){
	return stripDiacritics(m_ipa, diacritic);
    }
    public String stripTones(String tone){
	return stripTones(m_ipa, tone);
    }
    public String stripSuprasegmentals(String suprasegmental){
	return stripSuprasegmentals(m_ipa, suprasegmental);
    }
    public String stripVowels(String vowel){
	return stripVowels(m_ipa, vowel);
    }
    public String stripPlosives(String plosive){
	return stripPlosives(m_ipa, plosive);
    }
    public String stripNasals(String nasal){
	return stripNasals(m_ipa, nasal);
    }
    public String stripTrills(String trill){
	return stripTrills(m_ipa, trill);
    }
    public String stripFlaps(String flap){
	return stripFlaps(m_ipa, flap);
    }
    public String stripFricatives(String fricative){
	return stripFricatives(m_ipa, fricative);
    }
    public String stripLFricatives(String lfricative){
	return stripLFricatives(m_ipa, lfricative);
    }
    public String stripApproximants(String approximant){
	return stripApproximants(m_ipa, approximant);
    }
    public String stripLApproximants(String lapproximant){
	return stripLApproximants(m_ipa, lapproximant);
    }
    public String stripClicks(String click){
	return stripClicks(m_ipa, click);
    }
    public String stripImplosives(String implosive){
	return stripImplosives(m_ipa, implosive);
    }


    public String stripDiacritics(){
	return stripDiacritics(m_ipa, "");
    }
    public String stripTones(){
	return stripTones(m_ipa, "");
    }
    public String stripSuprasegmentals(){
	return stripSuprasegmentals(m_ipa, "");
    }
    public String stripVowels(){
	return stripVowels(m_ipa, "");
    }
    public String stripPlosives(){
	return stripPlosives(m_ipa, "");
    }
    public String stripNasals(){
	return stripNasals(m_ipa, "");
    }
    public String stripTrills(){
	return stripTrills(m_ipa, "");
    }
    public String stripFlaps(){
	return stripFlaps(m_ipa, "");
    }
    public String stripFricatives(){
	return stripFricatives(m_ipa, "");
    }
    public String stripLFricatives(){
	return stripLFricatives(m_ipa, "");
    }
    public String stripApproximants(){
	return stripApproximants(m_ipa, "");
    }
    public String stripLApproximants(){
	return stripLApproximants(m_ipa, "");
    }
    public String stripClicks(){
	return stripClicks(m_ipa, "");
    }
    public String stripImplosives(){
	return stripImplosives(m_ipa, "");
    }

    /* phonetische formeln:
     * ideen:
     *  1) reduktion auf "einfache laute": z.b. {a,e,i,o,u}=Vokal; {m,n}=Nasal
     *     pro: einfach, schnell (suche ber groe datenstze)
     *     contra: evtl. zu wenig complex
     *  2) jeder laut wird als mehrdimensionaler vektor dargestellt: 
     *       z.B. 
     *     pro: kann alles abbilden
     *     contra: viel zu kompliziert (bruchte viel know-how zum aufspannen des datenraumes, wie setzt man lautschrift in vektorfeld um ?, ), vermutlich langsam (bei groen datenmengen)
     *  3) neuronales netzwerk
     *     pro: sebstorganisation der daten, schnell
     *     contra: unischerheit ob es funktioniert, datenaufbereitungdamit es tatschlich funktioniert
     *             hnliche probleme wie bei punkt-2
     *
     */

    /*
     * complexity of the phonetic formula:
     * float: gradually changeable and extensible...
     * - ranges between 0 (simplest) to 1 (most complex)
     * - other complexities:
     *     no_spacing, no_diacritics, sound_groups (vowels, plosives,...),...
     */

    /**
     * complexity of a phonetic formula:
     * 
     */
    public static final int FORMULA_0  = 0;
    
    /**
     * complexity of a phonetic formula:
     * 
     */
    public static final int STRIP_VOWELS  = 1<<0;
    public static final int STRIP_PLOSIVES  = 1<<1;
    public static final int STRIP_NASALS  = 1<<2;
    public static final int STRIP_TRILLS  = 1<<3;
    public static final int STRIP_FLAPS  = 1<<4;
    public static final int STRIP_FRICATIVES  = 1<<5;
    public static final int STRIP_LFRICATIVES  = 1<<6;
    public static final int STRIP_ALL_FRICATIVES  = STRIP_FRICATIVES|STRIP_LFRICATIVES;

    public static final int STRIP_APPROXIMANTS  = 1<<7;
    public static final int STRIP_LAPPROXIMANTS  = 1<<8;
    public static final int STRIP_ALL_APPROXIMANTS  = STRIP_APPROXIMANTS|STRIP_LAPPROXIMANTS;
    public static final int STRIP_CLICKS  = 1<<9;
    public static final int STRIP_IMPLOSIVES  = 1<<10;
    public static final int STRIP_CONSONANTS  = STRIP_PLOSIVES|
	STRIP_NASALS|
	STRIP_TRILLS|
	STRIP_FLAPS|
	STRIP_ALL_FRICATIVES|
	STRIP_ALL_APPROXIMANTS|
	STRIP_CLICKS|
	STRIP_IMPLOSIVES;

    public static final int STRIP_DIACRITICS  = 1<<11;
    public static final int STRIP_SUPRASEGMENTALS  = 1<<12;
    public static final int STRIP_TONES  = 1<<13;
    public static final int STRIP_EXTRAS = STRIP_DIACRITICS|STRIP_SUPRASEGMENTALS|STRIP_TONES;


    public static final int STRIP_ALL = STRIP_VOWELS|STRIP_CONSONANTS|STRIP_EXTRAS;

    private static String getFormula(String ipa, int mask){
	String form=new String(ipa);

	if ((mask&STRIP_DIACRITICS)==STRIP_DIACRITICS)
	    form=stripDiacritics(form, "");
	if ((mask&STRIP_SUPRASEGMENTALS)==STRIP_SUPRASEGMENTALS)
	    form=stripSuprasegmentals(form, "");
	if ((mask&STRIP_TONES)==STRIP_TONES)
	    form=stripTones(form, "!");
	if ((mask&STRIP_PLOSIVES)==STRIP_PLOSIVES)
	    form=stripPlosives(form, "P");
	if ((mask&STRIP_NASALS)==STRIP_NASALS)
	    form=stripNasals(form, "N");
	if ((mask&STRIP_TRILLS)==STRIP_TRILLS)
	    form=stripTrills(form, "T");
	if ((mask&STRIP_FLAPS)==STRIP_FLAPS)
	    form=stripFlaps(form, "R");
	if ((mask&STRIP_FRICATIVES)==STRIP_FRICATIVES)
	    form=stripFricatives(form, "F");
	if ((mask&STRIP_LFRICATIVES)==STRIP_LFRICATIVES)
	    form=stripLFricatives(form, "F");
	if ((mask&STRIP_APPROXIMANTS)==STRIP_APPROXIMANTS)
	    form=stripApproximants(form, "A");
	if ((mask&STRIP_LAPPROXIMANTS)==STRIP_LAPPROXIMANTS)
	    form=stripLApproximants(form, "A");
	if ((mask&STRIP_CLICKS)==STRIP_CLICKS)
	    form=stripClicks(form, "C");
	if ((mask&STRIP_IMPLOSIVES)==STRIP_IMPLOSIVES)
	    form=stripImplosives(form, "I");
	if ((mask&STRIP_VOWELS)==STRIP_VOWELS)
	    form=stripVowels(form, "V");

	return form;
    }

    private static String getFormula(String ipa){
	return getFormula(ipa, STRIP_EXTRAS); //STRIP_ALL);
    }

    /**
     * convert the IPA-String into a formula, regardless of delimiters...
     * LATER: think about improving this
     */
    public String getFormula(){
	return getFormula(m_ipa);
    }

    /**
     * extract the IPA-encoded data (within "startID" and "stopID") and 
     * convert each part into a phonetic formula
     * @param startID the delimiter that marks the beginning of a SAMPA-encoded sequence in the org String
     * @param stopID the delimiter that marks the ending of a SAMPA-encoded sequence in the org String
     * @return the String-array holding the phonetic formulae for each part of the IPA-encoded data
     */
    public String[] getFormulae(String startID, String stopID){
	int start=0;
	int stop=0;

	final int startLen=startID.length();
	final int stopLen =stopID.length();


	int count=0;

	start=m_ipa.indexOf(startID);
	stop=m_ipa.indexOf(stopID, start+startLen);

	/* count how many formulae we will return */
	while(start>=0){
	    count++;
	    start=(stop>0)?m_ipa.indexOf(startID, stop+stopLen):-1;
	    stop=m_ipa.indexOf(stopID, start+startLen);
	}

	/* create an array that can hold all the formulae */
	String[] result = new String[count];
	String substring;


	/* strip the formulae from the IPA-string */
	start=m_ipa.indexOf(startID);
	stop=m_ipa.indexOf(stopID, start+startLen);

	for(int i=0; i<count; i++){
	    substring=(stop<0)?m_ipa.substring(start+startLen):m_ipa.substring(start+startLen, stop);
	    result[i]=getFormula(substring);
	    start=m_ipa.indexOf(startID, stop+stopLen);
	    stop =m_ipa.indexOf(stopID, start+startLen);
	}

	return result;	
    }
   /**
     * extract the IPA-encoded data (within "startID" and "stopID")
     * @param startID the delimiter that marks the beginning of a SAMPA-encoded sequence in the org String
     * @param stopID the delimiter that marks the ending of a SAMPA-encoded sequence in the org String
     * @return the String-array holding the sub-blocks of IPA-encoded data
     */
    public String[] getSubTranscriptions(String startID, String stopID){
	int start=0;
	int stop=0;

	final int startLen=startID.length();
	final int stopLen =stopID.length();

	int count=0;

	start=m_ipa.indexOf(startID);
	stop=m_ipa.indexOf(stopID, start+startLen);

	/* count how many blocks we will return */
	while(start>=0){
	    count++;
	    start=(stop>0)?m_ipa.indexOf(startID, stop+stopLen):-1;
	    stop=m_ipa.indexOf(stopID, start+startLen);
	}

	/* create an array that can hold all the blocks */
	String[] result = new String[count];
	String substring;


	/* strip the blocks from the IPA-string */
	start=m_ipa.indexOf(startID);
	stop=m_ipa.indexOf(stopID, start+startLen);

	for(int i=0; i<count; i++){
	    result[i]=(stop<0)?m_ipa.substring(start+startLen):m_ipa.substring(start+startLen, stop);
	    start=m_ipa.indexOf(startID, stop+stopLen);
	    stop =m_ipa.indexOf(stopID, start+startLen);
	}

	return result;
    }

    /**
     * partly convert a given String vom SAMPA to IPA
     * the part that is to be converted is delimited by startID and stopID
     * @param org the String encoded in SAMPA
     * @param startID the delimiter that marks the beginning of a SAMPA-encoded sequence in the org String
     * @param stopID the delimiter that marks the ending of a SAMPA-encoded sequence in the org String
     * @return the String encoded in IPA
     */
    private static String sampa2ipaX(String org, String startID, String stopID){
	int start=0;
	int stop=0;
	start=org.indexOf(startID);
	stop=org.indexOf(stopID, start);
	if (start<0)return org;
	String result=org.substring(0, start);
	String sampa;

	while(start>=0){
	    sampa = (stop<0)?org.substring(start):org.substring(start, stop);
	    result = result + sampa2ipa(sampa);
	    //System.out.println("formula: "+getFormula(sampa2ipa(sampa)));
	    if (stop<0)return result;
	    start=org.indexOf(startID, stop);
	    result= result+((start<0)?org.substring(stop):org.substring(stop, start));
	    stop =org.indexOf(stopID, start);
	}
	return result;
    }
    /**
     * partly convert a given String vom IPA to SAMPA
     * the part that is to be converted is delimited by startID and stopID
     * @param org the String encoded in IPA
     * @param startID the delimiter that marks the beginning of a IPA-encoded sequence in the org String
     * @param stopID the delimiter that marks the ending of a IPA-encoded sequence in the org String
     * @return the String encoded in SAMPA
     */
    private static String ipa2sampaX(String org, String startID, String stopID){
	int start=0;
	int stop=0;
	start=org.indexOf(startID);
	stop=org.indexOf(stopID, start);
	if (start<0)return org;
	String result=org.substring(0, start);
	String ipa;

	while(start>=0){
	    ipa = (stop<0)?org.substring(start):org.substring(start, stop);
	    result = result + ipa2sampa(ipa);
	    if (stop<0)return result;
	    start=org.indexOf(startID, stop);
	    result= result+((start<0)?org.substring(stop):org.substring(stop, start));
	    stop =org.indexOf(stopID, start);
	}
	return result;
    }
    /**
     * partly convert a given String vom SAMPA to IPA
     * the part that is to be converted is delimited by "[[" and "]]"
     * @param org the String encoded in SAMPA
     * @return the String encoded in IPA
     */
    private static String sampa2ipaX(String org){
	return sampa2ipaX(org, "[[", "]]");
    }
    /**
     * partly convert a given String vom IPA to SAMPA
     * the part that is to be converted is delimited by "[[" and "]]"
     * @param org the String encoded in IPA
     * @return the String encoded in SAMPA
     */
    private static String ipa2sampaX(String org){
	return ipa2sampaX(org, "[[", "]]");
    }
    /**
     * in TIPA (that is: TeX) some character-modifiers have to be pre-prended while others are post-pended.
     * this function finds out, which order has to be used for a specified characters
     * @param c the character to be tested
     * @return <code>true</code> if characters have to be swapped
     * <code>false</code> otherwise
     */
    private static boolean swapTipaEntity(char c){
	if (c>0x300 && c<=0x033d)
	    if (c==0x31a || c==0x321 || c==0x0322)return false;
	    else return true;
	return false;
    }

    /**
     * in TIPA (that is: TeX) some character-modifiers have to enclose a group of characters.
     * this function finds out how many "entities" before and after the given modifier have to be enclosed;
     * @param s the modifier-string to be tested
     * @return <code>null</code> is the given entity does not enclose a group
     */
    private static TIPAenvironment groupTipaEntity(String s){
        if ('\u0000'==s.charAt(0)) {
            String env=s.substring(1);
            if(env.equals("\\t"))
                return new TIPAenvironment(env, 1, 1);
            if(env.equals("\\t*"))
                return new TIPAenvironment(env, 1, 1);
        }        
	return null;
    }


    private static class TIPAenvironment {
        public int pre, post;
        public String entity;
        public TIPAenvironment(String entity, int pre, int post){
            this.pre=pre;
            this.post=post;
            this.entity=entity;
        }
    }


    /**
     * convert a unicode IPA-character to its TIPA-representation
     * @param ipachar the unicode-encoded IPA-character
     * @return the TIPA-representation of the IPA-character
     */
    private static String TipaChar(char ipachar){
	switch (ipachar){
	case 0x00e6: return new String("\\ae ");
	case 0x00e7: return new String("\\c{c}");
	case 0x00f0: return new String("D");
	case 0x00f8: return new String("\\o ");
	case 0x0127: return new String("\\textcrh ");
	case 0x014b: return new String("N");
	case 0x0153: return new String("\\oe ");
	case 0x0180: return new String("\\textcrb ");
	case 0x0188: return new String("\\texthtc ");
	case 0x0195: return new String("\\texthvlig ");
	case 0x0199: return new String("\\texthtk ");
	case 0x019a: return new String("\\textbarl ");
	case 0x019b: return new String("\\textcrlambda ");
	case 0x019e: return new String("\\textnrleg ");
	case 0x01a5: return new String("\\texthtp ");
	case 0x01ab: return new String("\\textlhookt ");
	case 0x01ad: return new String("\\texthtt ");
	case 0x01b9: return new String("\\textrevyogh ");
	case 0x01c0: return new String("|");
	case 0x01c1: return new String("||");
	case 0x01c2: return new String("\\textdoublebarpipe ");
	case 0x01c3: return new String("!");
	case 0x0250: return new String("5");
	case 0x0251: return new String("A");
	case 0x0252: return new String("6");
	case 0x0253: return new String("\\!b");
	case 0x0254: return new String("O");
	case 0x0255: return new String("C");
	case 0x0256: return new String("\\:d");
	case 0x0257: return new String("\\!d");
	case 0x0258: return new String("9");
	case 0x0259: return new String("@");
	case 0x025a: return new String("\\textrhookschwa ");
	case 0x025b: return new String("E");
	case 0x025c: return new String("\\textrevepsilon ");
	case 0x025d: return new String("\\textrhookrevepsilon ");
	case 0x025e: return new String("\\textcloserevepsilon ");
	case 0x025f: return new String("\\textbardotlessj ");
	case 0x0260: return new String("\\!g");
	case 0x0261: return new String("g");
	case 0x0262: return new String("\\;G");
	case 0x0263: return new String("G");
	case 0x0264: return new String("7");
	case 0x0265: return new String("4");
	case 0x0266: return new String("H");
	case 0x0267: return new String("\\texththeng ");
	case 0x0268: return new String("1");
	case 0x0269: return new String("\\textiota ");
	case 0x026a: return new String("I");
	case 0x026b: return new String("\\textltilde ");
	case 0x026c: return new String("\\textbeltl ");
	case 0x026d: return new String("\\:l");
	case 0x026e: return new String("\\textlyoghlig ");
	case 0x026f: return new String("W");
	case 0x0270: return new String("\\textturnmrleg ");
	case 0x0271: return new String("M");
	case 0x0272: return new String("\\textltailn ");
	case 0x0273: return new String("\\:n");
	case 0x0274: return new String("\\;N");
	case 0x0275: return new String("8");
	case 0x0276: return new String("\\OE ");
	case 0x0277: return new String("\\textcloseomega ");
	case 0x0278: return new String("F");
	case 0x0279: return new String("\\*r");
	case 0x027a: return new String("\\textturnlonglegr ");
	case 0x027b: return new String("\\:R");
	case 0x027c: return new String("\\textlonglegr ");
	case 0x027d: return new String("\\:r");
	case 0x027e: return new String("R");
	case 0x0280: return new String("\\;R");
	case 0x0281: return new String("K");
	case 0x0282: return new String("\\:s");
	case 0x0283: return new String("S");
	case 0x0284: return new String("\\!j");
	    //	case 0x0284: return new String("\\textdoublebaresh ");
	case 0x0286: return new String("\\textctesh ");
	case 0x0287: return new String("\\*t");
	case 0x0288: return new String("\\:t");
	case 0x0289: return new String("0");
	case 0x028a: return new String("U");
	case 0x028b: return new String("V");
	case 0x028c: return new String("2");
	case 0x028d: return new String("\\*w");
	case 0x028e: return new String("L");
	case 0x028f: return new String("Y");
	case 0x0290: return new String("\\:z");
	case 0x0291: return new String("\\textctz ");
	case 0x0292: return new String("Z");
	case 0x0293: return new String("\\textctyogh ");
	case 0x0294: return new String("P");
	case 0x0295: return new String("Q");
	case 0x0296: return new String("\\textinvglotstop ");
	case 0x0297: return new String("\\textstretchc ");
	case 0x0298: return new String("\\!o");
	case 0x0299: return new String("\\;B");
	case 0x029a: return new String("\\textcloseepsilon ");
	case 0x029b: return new String("\\!G");
	case 0x029c: return new String("\\;H");
	case 0x029d: return new String("J");
	case 0x029e: return new String("\\*k");
	case 0x029f: return new String("\\;L");
	case 0x02a0: return new String("\\texthtq ");
	case 0x02a1: return new String("\\textbarglotstop ");
	case 0x02a2: return new String("\\textbarrevglotstop ");
	case 0x02a3: return new String("\\textdzlig ");
	case 0x02a4: return new String("\\textdyoghlig ");
	case 0x02a5: return new String("\\textdctzlig ");
	case 0x02a6: return new String("\\texttslig ");
	case 0x02a7: return new String("\\textteshlig ");
	case 0x02a8: return new String("\\texttctclig ");
	case 0x02b0: return new String("\\super{h}");
	case 0x02b1: return new String("\\super{H}");
	case 0x02b2: return new String("\\super{j}");
	case 0x02b3: return new String("\\super{r}");
	case 0x02b4: return new String("\\super{*r}");
	case 0x02b5: return new String("\\super{:R}");
	case 0x02b6: return new String("\\super{K}");
	case 0x02b7: return new String("\\super{w}");
	case 0x02b8: return new String("\\super{y}");
	case 0x02bc: return new String("'");
	case 0x02c0: return new String("\\textraiseglotstop ");
	case 0x02c1: return new String("\\super{Q}");
	case 0x02c8: return new String("\"");
	case 0x02cc: return new String("\\textsecstress ");
	case 0x02d0: return new String(":");
	case 0x02d1: return new String(";");
	case 0x02de: return new String("\\textrhoticity ");
	case 0x02e0: return new String("\\super{G}");
	case 0x02e1: return new String("\\super{l}");
	case 0x02e2: return new String("\\super{s}");
	case 0x02e3: return new String("\\super{x}");
	case 0x02e4: return new String("\\super{Q}");
	case 0x02e5: return new String("\\tone{55}");
	case 0x02e6: return new String("\\tone{44}");
	case 0x02e7: return new String("\\tone{33}");
	case 0x02e8: return new String("\\tone{22}");
	case 0x02e9: return new String("\\tone{11}");
	case 0x0300: return new String("\\`");
	case 0x0301: return new String("\\'");
	case 0x0302: return new String("\\^");
	case 0x0303: return new String("\\~");
	case 0x0304: return new String("\\=");
	case 0x0306: return new String("\\u ");
	case 0x0308: return new String("\\\"");
	case 0x030a: return new String("\\r ");
	case 0x030b: return new String("\\H ");
	case 0x030c: return new String("\\v ");
	case 0x030f: return new String("\\H*");
	case 0x0318: return new String("\\|<");
	case 0x0319: return new String("\\|>");
	case 0x031a: return new String("\\textcorner ");
	case 0x031c: return new String("\\|(");
	case 0x031d: return new String("\\|'");
	case 0x031e: return new String("\\|`");
	case 0x031f: return new String("\\|+");
	case 0x0320: return new String("\\=*");
	case 0x0321: return new String("\\textpalhook ");
	case 0x0322: return new String("\\textrthook ");
	case 0x0323: return new String("\\.*");
	case 0x0324: return new String("\\\"*");
	case 0x0325: return new String("\\r*");
	case 0x0329: return new String("\\s ");
	case 0x032a: return new String("\\|[");
	case 0x032b: return new String("\\|w");
	case 0x032c: return new String("\\v*");
	case 0x032f: return new String("\\textsubarch ");
	case 0x0330: return new String("\\~*");
	case 0x0334: return new String("\\textsuperimposetilde ");
	case 0x0339: return new String("\\|)");
	case 0x033a: return new String("\\|]");
	case 0x033b: return new String("\\textsubsquare ");
	case 0x033c: return new String("\\|m");
	case 0x033d: return new String("\\|x");
	case 0x03b2: return new String("B");
	case 0x03b8: return new String("T");
	case 0x03c7: return new String("X");
	case 0x207f: return new String("\\super{n}");
	case 0x2191: return new String("\\textupstep ");
	case 0x2193: return new String("\\textdownstep ");
	case 0x2197: return new String("\\textglobrise ");
	case 0x2198: return new String("\\textglobfall ");
	case 0x0067: return new String("\\textg "); // g
	case 0x0020: return new String(" "); // <space>

            /* these are grouping entities, they need special handling: e.g 'a_(b' --> '\t*{ab}';
             * we first prefix a special character, which will then be stripped on final expansion.
             * the special-char servers 2 purposes: easy detection of special handling + flagging whether this has already been processed
             */
        case 0x0361: return new String("\u0000\\t"); 
        case 0x035c: return new String("\u0000\\t*"); 
	default:
	    if (ipachar<129 && ipachar>31)return new String(ipachar+"");
	}
	return null;
    }
    /**
     * convert a transcription String from IPA(unicode) to TIPA(TeX)
     * @param ipa the IPA-encoded String
     * @return the TIPA-encoded String
     */
    private static String ipa2tipa(String ipa){
        if(ipa==null)return null;

        String[]sa_template=new String[1];
        java.util. ArrayList<String> tipa = new java.util.ArrayList<String>();
        for(int i=0; i<ipa.length(); i++) {
            char c=ipa.charAt(i);
            if (swapTipaEntity(c)){
                int index=tipa.size()-1;
                String org = (index<0)?"":((String)tipa.get(index));
                tipa.set(index, new String(TipaChar(c)+"{"+org+"}"));
            } else {
                tipa.add(TipaChar(c));
            }
        }
        if(tipa.size()<=0)
            return null;

        TIPAenvironment env;
        for(int index=0; index<tipa.size(); index++) {
            String entity=(String)tipa.get(index);
            env=groupTipaEntity(entity);
            if(env!=null) {
                int start=index-env.pre;
                int stop=index +env.post+1;
                if(start<0)
                    start=0;
                if(stop>tipa.size())
                    stop=tipa.size();

                tipa.add(stop, "}");
                tipa.remove(index);
                tipa.add(start, env.entity+"{");
            }
        }

        return at.iem.tools.StringX.merge((String[])tipa.toArray(sa_template));
    }

    /**
     * partly convert a transcription String from IPA(unicode) to TIPA(TeX)
     * the parts that should be converted are delimited by <code>startID</code> and <code>stopID</code>
     * @param org the IPA-encoded String
     * @param startID the starting delimiter
     * @param stopID the ending delimiter
     * @return the TIPA-encoded String
     */
    private static String ipa2tipa(String org, String startID, String stopID){
	int start=0;
	int stop=0;
        int start_len=startID.length();
        int stop_len=stopID.length();
	start=org.indexOf(startID);
	stop=org.indexOf(stopID, start);
	if (start<0)return org;
	String result=org.substring(0, start);
	String ipa;

	while(start>=0){
	    ipa = (stop<0)?org.substring(start+start_len):org.substring(start+start_len, stop);
	    result = result + startID + ipa2tipa(ipa) + stopID;
	    if (stop<0)return result;
	    start=org.indexOf(startID, stop);
	    result= result+((start<0)?org.substring(stop+stop_len):org.substring(stop+stop_len, start));
	    stop =org.indexOf(stopID, start);
	}
	return result;
    }
    /**
     * replace a substring of a string by another string
     * @param org the original string
     * @param from the substring to be replaced
     * @param to the replacing substring
     * @return the replaced string
     */
    private static String replace(String org, String from, String to){
	return at.iem.tools.StringX.replace(org, from, to);
    }

    /**
     * some fonts do not render the ligatures "lz", "ls",... correctly.
     * so we replace them by separate characters
     * @param ipa the String to be simplified
     * @return the simplified String
     */
    public static String simplify(String ipa){
	// some Unicode-fonts do not display following characters
	if (ipa==null)return null;
	String ipa2 = new String(ipa);
	ipa2 = at.iem.tools.StringX.replace(ipa2, "\u02ab", "lz");      // lz
	ipa2 = at.iem.tools.StringX.replace(ipa2, "\u02aa", "ls");      // ls
	ipa2 = at.iem.tools.StringX.replace(ipa2, "\u02a9", "f\u014b"); // fN
	return ipa2;
    }

    /**
     * some fonts do not render the ligatures "lz", "ls",... correctly.
     * so we replace them by separate characters
     * @param ipa the IPA to be simplified
     * @return the simplified String
     */
    public static String simplify(IPA ipa){return simplify(ipa.toString());}

    private static String regex_replaceHelper(String s, String pattern){
	String result = new String(s);
        String chars[]=getGroup(pattern);
	String to="";
	for(int i=0; i<chars.length; i++)to+=chars[i];

	result=at.iem.tools.StringX.replace(result, "\\p{"+pattern+"}", "["+to+"]");
	result=at.iem.tools.StringX.replace(result, "\\P{"+pattern+"}", "[^"+to+"]");
	return result;
    }
    private static String regex_replaceHelper(String s, String pattern, String[]chars){
	String result = new String(s);
	String to="";
	for(int i=0; i<chars.length; i++)to+=chars[i];

	result=at.iem.tools.StringX.replace(result, "\\p{"+pattern+"}", "["+to+"]");
	result=at.iem.tools.StringX.replace(result, "\\P{"+pattern+"}", "[^"+to+"]");
	return result;
    }
    private static String regex_replaceHelper(String s, String pattern, String[]chars1, String[]chars2){
	String result = new String(s);
	String to="";
	for(int i=0; i<chars1.length; i++)to+=chars1[i];
	for(int i=0; i<chars2.length; i++)to+=chars2[i];

	result=at.iem.tools.StringX.replace(result, "\\p{"+pattern+"}", "["+to+"]");
	result=at.iem.tools.StringX.replace(result, "\\P{"+pattern+"}", "[^"+to+"]");
	return result;
    }

    /**
     * define some regex-character-classes for IPA:
     * classes are: 
     * <em>IPAdiacritic, IPAtone, IPAsuprasegmental, IPAvowel, IPAconsonant, 
     IPAplosive, IPAnasal, IPAtrill, IPAflap, IPAtap, IPAfricative, IPAapproximant,
     IPAclick, IPAimplosive,
     IPAbilabial, IPAlabiodental, IPAdental, IPAalveolar, IPApostalveolar, IPAretroflex, IPApalatal, IPAvelar, IPAuvular, IPApharyngal, IPAglottal, IPAepiglottal</em>
     */

    public static String regex(String Pattern){
	String result = new String(Pattern);
	result=regex_replaceHelper(result, "IPAdiacritic", DIACRITICS);
	result=regex_replaceHelper(result, "IPAtone", TONES);
	result=regex_replaceHelper(result, "IPAsuprasegmental", SUPRASEGMENTALS);
	result=regex_replaceHelper(result, "IPAvowel", VOWELS);
	result=regex_replaceHelper(result, "IPAconsonant", CONSONANTS);

	result=regex_replaceHelper(result, "IPAplosive", PLOSIVES);
	result=regex_replaceHelper(result, "IPAnasal", NASALS);
	result=regex_replaceHelper(result, "IPAtrill", TRILLS);
	result=regex_replaceHelper(result, "IPAflap", FLAPS);
	result=regex_replaceHelper(result, "IPAtap", FLAPS);
	result=regex_replaceHelper(result, "IPAlfricative", LATERAL_FRICATIVES);
	result=regex_replaceHelper(result, "IPAfricative", FRICATIVES, LATERAL_FRICATIVES);
	result=regex_replaceHelper(result, "IPAlapproximant", LATERAL_APPROXIMANTS);
	result=regex_replaceHelper(result, "IPAapproximant", APPROXIMANTS, LATERAL_APPROXIMANTS);
	result=regex_replaceHelper(result, "IPAlateral", LATERAL_FRICATIVES, LATERAL_APPROXIMANTS);
	result=regex_replaceHelper(result, "IPAclick", CLICKS);
	result=regex_replaceHelper(result, "IPAimplosive", IMPLOSIVES);
	result=regex_replaceHelper(result, "IPAbilabial", BILABIALS);
	result=regex_replaceHelper(result, "IPAlabiodental", LABIODENTALS);
	result=regex_replaceHelper(result, "IPAdental", DENTALS);
	result=regex_replaceHelper(result, "IPAalveolar", ALVEOLARS);
	result=regex_replaceHelper(result, "IPApostalveolar", POSTALVEOLARS);
	result=regex_replaceHelper(result, "IPAretroflex", RETROFLEXS);
	result=regex_replaceHelper(result, "IPApalatal", PALATALS);
	result=regex_replaceHelper(result, "IPAvelar", VELARS);
	result=regex_replaceHelper(result, "IPAuvular", UVULARS);
	result=regex_replaceHelper(result, "IPApharyngal", PHARYNGEALS);
	result=regex_replaceHelper(result, "IPAglottal", GLOTTALS);
	result=regex_replaceHelper(result, "IPAepiglottal", EPIGLOTTALS);
	result=regex_replaceHelper(result, "IPAcombined", COMBINED);

	result=regex_replaceHelper(result, "IPAejective", COMBINED);
	
	return result;
    }

    public static String[] getGroup(String id){
	if(id.equals("IPAdiacritic"))return DIACRITICS;
	if(id.equals("IPAtone"))return TONES;
	if(id.equals("IPAsuprasegmental"))return SUPRASEGMENTALS;
	if(id.equals("IPAvowel"))return VOWELS;
	if(id.equals("IPAconsonant"))return CONSONANTS;

	if(id.equals("IPAplosive"))return PLOSIVES;
	if(id.equals("IPAnasal"))return NASALS;
	if(id.equals("IPAtrill"))return TRILLS;
	if(id.equals("IPAflap"))return FLAPS;
	if(id.equals("IPAtap"))return FLAPS;
	if(id.equals("IPAclick"))return CLICKS;
	if(id.equals("IPAimplosive"))return IMPLOSIVES;
	if(id.equals("IPAbilabial"))return BILABIALS;
	if(id.equals("IPAlabiodental"))return LABIODENTALS;
	if(id.equals("IPAdental"))return DENTALS;
	if(id.equals("IPAalveolar"))return ALVEOLARS;
	if(id.equals("IPApostalveolar"))return POSTALVEOLARS;
	if(id.equals("IPAretroflex"))return RETROFLEXS;
	if(id.equals("IPApalatal"))return PALATALS;
	if(id.equals("IPAvelar"))return VELARS;
	if(id.equals("IPAuvular"))return UVULARS;
	if(id.equals("IPApharyngal"))return PHARYNGEALS;
	if(id.equals("IPAglottal"))return GLOTTALS;
	if(id.equals("IPAepiglottal"))return EPIGLOTTALS;
	if(id.equals("IPAcombined"))return COMBINED;

	if(id.equals("IPAejective"))return EJECTIVES;

	if(id.equals("IPAlfricative"))return  LATERAL_FRICATIVES;
	if(id.equals("IPAlapproximant"))return  LATERAL_APPROXIMANTS;

	if(id.equals("IPAfricative")){
            return at.iem.tools.SetOperator.union(FRICATIVES, LATERAL_FRICATIVES);
        }
	if(id.equals("IPAapproximant")) {
            return at.iem.tools.SetOperator.union(APPROXIMANTS, LATERAL_APPROXIMANTS);
        }
	if(id.equals("IPAlateral")){
            return at.iem.tools.SetOperator.union(LATERAL_FRICATIVES, LATERAL_APPROXIMANTS);
        }
        System.out.println("unknown IPA group: '"+id+"'");
        return null;
    }

    /**
     * assume that the current ipa-string is wrongly encoded to look correctly on Microsoft fonts.
     * this function will then fix all the Microsoft font bugs
     */
    public void fixMS2Unicode() {
        m_ipa=fixMS2Unicode(m_ipa);
    }


    private static String[] doublewidthdiacritics = {
        "\u0360",
        "\u0361",
        "\u0362",
   };

    /**
     * make a correct unicode string look correctly with Microsoft fonts
     */
    public static String fixUnicode2MS(String s) {
        String result=new String(s);
        for(int i=0; i<doublewidthdiacritics.length; i++) {
            String dia=doublewidthdiacritics[i];
            /*
            String oldS="(.)(.)"+dia;
            String newS="$1"+dia+"$2";
            */
            String oldS="(.)"+dia+"(.)";
            String newS="$1$2"+dia;

            //System.out.println("fixMSDoubleWidthDiacritics: "+oldS+" -> "+newS);
            result=result.replaceAll(oldS, newS);
        }
        //System.out.println("fixedMSDoubleWidthDiacritics: "+s+" --> "+result);

        return result;
    }
    /**
     * make a unicode string that looks correctly with Microsoft fonts to a string that is correctly encoded
     */
    public static String fixMS2Unicode(String s) {
        String result=new String(s);
        for(int i=0; i<doublewidthdiacritics.length; i++) {
            String dia=doublewidthdiacritics[i];
            String oldS="(.)(.)"+dia;
            String newS="$1"+dia+"$2";

            //System.out.println("fixMSDoubleWidthDiacritics: "+oldS+" -> "+newS);
            result=result.replaceAll(oldS, newS);
        }
        //System.out.println("fixedMSDoubleWidthDiacritics: "+s+" --> "+result);

        return result;
    }

    /**
     * compare 2 IPA-strings
     */
    public boolean equals(IPA other) {
        if(null==other) return false;
        return m_ipa.equals(other.m_ipa);
    }
    public boolean equals(String other) {
        if(null==other) return false;
        return m_ipa.equals(other);
    }
}
